@@ -40,24 +40,26 @@ from cinder import exception
 from cinder.i18n import _
 from cinder import objects
 from cinder.objects import fields
-from cinder.volume import utils as volume_utils
+from cinder.volume import volume_utils
 
 if sys.platform == 'win32':
     from os_win import utilsfactory as os_win_utilsfactory
 
 LOG = logging.getLogger(__name__)
 
-chunkedbackup_service_opts = [
+backup_opts = [
     cfg.StrOpt('backup_compression_algorithm',
                default='zlib',
+               ignore_case=True,
                choices=['none', 'off', 'no',
                         'zlib', 'gzip',
-                        'bz2', 'bzip2'],
-               help='Compression algorithm (None to disable)'),
+                        'bz2', 'bzip2',
+                        'zstd'],
+               help='Compression algorithm ("none" to disable)'),
 ]
 
 CONF = cfg.CONF
-CONF.register_opts(chunkedbackup_service_opts)
+CONF.register_opts(backup_opts)
 
 
 # Object writer and reader returned by inheriting classes must not have any
@@ -91,6 +93,9 @@ class ChunkedBackupDriver(driver.BackupDriver):
             elif algorithm.lower() in ('bz2', 'bzip2'):
                 import bz2 as compressor
                 result = compressor
+            elif algorithm.lower() == 'zstd':
+                import zstd as compressor
+                result = compressor
             else:
                 result = None
             if result:
@@ -152,7 +157,7 @@ class ChunkedBackupDriver(driver.BackupDriver):
 
     @abc.abstractmethod
     def get_object_writer(self, container, object_name, extra_metadata=None):
-        """Returns a writer object which stores the chunk data in backup repository.
+        """Returns a writer object which stores the chunk data.
 
         The object returned should be a context handler that can be used in a
         "with" context.
@@ -427,7 +432,12 @@ class ChunkedBackupDriver(driver.BackupDriver):
                              object_list,
                              volume_meta,
                              extra_metadata)
-        backup.object_count = object_id
+        # NOTE(whoami-rajat) : The object_id variable is used to name
+        # the backup objects and hence differs from the object_count
+        # variable, therefore the increment of object_id value in the last
+        # iteration of _backup_chunk() method shouldn't be reflected in the
+        # object_count variable.
+        backup.object_count = object_id - 1
         backup.save()
         LOG.debug('backup %s finished.', backup['id'])
 
@@ -711,9 +721,12 @@ class ChunkedBackupDriver(driver.BackupDriver):
                 LOG.debug('decompressing data using %s algorithm',
                           compression_algorithm)
                 decompressed = decompressor.decompress(body)
+                body = None  # Allow Python to free it
                 volume_file.write(decompressed)
+                decompressed = None  # Allow Python to free it
             else:
                 volume_file.write(body)
+                body = None  # Allow Python to free it
 
             # force flush every write to avoid long blocking write on close
             volume_file.flush()
