@@ -32,11 +32,13 @@ import stat
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_log import versionutils
 
 from cinder.backup import driver
 from cinder import exception
-from cinder.i18n import _LE, _
+from cinder.i18n import _
 from cinder import interface
+import cinder.privsep.path
 from cinder import utils
 
 LOG = logging.getLogger(__name__)
@@ -108,9 +110,7 @@ def _make_link(volume_path, backup_path, vol_id):
     """
 
     try:
-        utils.execute('ln', volume_path, backup_path,
-                      run_as_root=True,
-                      check_exit_code=True)
+        cinder.privsep.path.symlink(volume_path, backup_path)
     except processutils.ProcessExecutionError as exc:
         err = (_('backup: %(vol_id)s failed to create device hardlink '
                  'from %(vpath)s to %(bpath)s.\n'
@@ -250,9 +250,9 @@ def _cleanup_device_hardlink(hardlink_path, volume_path, volume_id):
                       hardlink_path,
                       run_as_root=True)
     except processutils.ProcessExecutionError as exc:
-        LOG.error(_LE('backup: %(vol_id)s failed to remove backup hardlink '
-                      'from %(vpath)s to %(bpath)s.\n'
-                      'stdout: %(out)s\n stderr: %(err)s.'),
+        LOG.error('backup: %(vol_id)s failed to remove backup hardlink '
+                  'from %(vpath)s to %(bpath)s.\n'
+                  'stdout: %(out)s\n stderr: %(err)s.',
                   {'vol_id': volume_id,
                    'vpath': volume_path,
                    'bpath': hardlink_path,
@@ -266,11 +266,28 @@ class TSMBackupDriver(driver.BackupDriver):
 
     DRIVER_VERSION = '1.0.0'
 
-    def __init__(self, context, db_driver=None):
-        super(TSMBackupDriver, self).__init__(context, db_driver)
+    def __init__(self, context, db=None):
+        super(TSMBackupDriver, self).__init__(context, db)
         self.tsm_password = CONF.backup_tsm_password
         self.volume_prefix = CONF.backup_tsm_volume_prefix
 
+    @staticmethod
+    def get_driver_options():
+        return tsm_opts
+
+    def check_for_setup_error(self):
+        versionutils.report_deprecated_feature(
+            LOG,
+            "Cinder TSM Backup Driver is deprecated and will be removed "
+            "in Wallaby release. Please, migrate you backups to a supported "
+            "backend.")
+        required_flags = ['backup_share']
+        for flag in required_flags:
+            val = getattr(CONF, flag, None)
+            if not val:
+                raise exception.InvalidConfigurationValue(option=flag,
+                                                          value=val)
+
     def _do_backup(self, backup_path, vol_id, backup_mode):
         """Perform the actual backup operation.
 
@@ -395,18 +412,6 @@ class TSMBackupDriver(driver.BackupDriver):
                       'err': exc.stderr})
             LOG.error(err)
             raise exception.InvalidBackup(reason=err)
-        except exception.Error as exc:
-            err = (_('backup: %(vol_id)s failed to run dsmc '
-                     'due to invalid arguments '
-                     'on %(bpath)s.\n'
-                     'stdout: %(out)s\n stderr: %(err)s')
-                   % {'vol_id': backup.volume_id,
-                      'bpath': backup_path,
-                      'out': exc.stdout,
-                      'err': exc.stderr})
-            LOG.error(err)
-            raise exception.InvalidBackup(reason=err)
-
         finally:
             _cleanup_device_hardlink(backup_path, volume_path,
                                      backup.volume_id)
@@ -455,18 +460,6 @@ class TSMBackupDriver(driver.BackupDriver):
                       'err': exc.stderr})
             LOG.error(err)
             raise exception.InvalidBackup(reason=err)
-        except exception.Error as exc:
-            err = (_('restore: %(vol_id)s failed to run dsmc '
-                     'due to invalid arguments '
-                     'on %(bpath)s.\n'
-                     'stdout: %(out)s\n stderr: %(err)s')
-                   % {'vol_id': volume_id,
-                      'bpath': restore_path,
-                      'out': exc.stdout,
-                      'err': exc.stderr})
-            LOG.error(err)
-            raise exception.InvalidBackup(reason=err)
-
         finally:
             _cleanup_device_hardlink(restore_path, volume_path, volume_id)
 
@@ -474,7 +467,7 @@ class TSMBackupDriver(driver.BackupDriver):
                   {'backup_id': backup.id,
                    'volume_id': volume_id})
 
-    def delete(self, backup):
+    def delete_backup(self, backup):
         """Delete the given backup from TSM server.
 
         :param backup: backup information for volume
@@ -508,29 +501,15 @@ class TSMBackupDriver(driver.BackupDriver):
                       'err': exc.stderr})
             LOG.error(err)
             raise exception.InvalidBackup(reason=err)
-        except exception.Error as exc:
-            err = (_('delete: %(vol_id)s failed to run dsmc '
-                     'due to invalid arguments with '
-                     'stdout: %(out)s\n stderr: %(err)s')
-                   % {'vol_id': backup.volume_id,
-                      'out': exc.stdout,
-                      'err': exc.stderr})
-            LOG.error(err)
-            raise exception.InvalidBackup(reason=err)
-
         success = _check_dsmc_output(out, delete_attrs)
         if not success:
             # log error if tsm cannot delete the backup object
             # but do not raise exception so that cinder backup
             # object can be removed.
-            LOG.error(_LE('delete: %(vol_id)s failed with '
-                          'stdout: %(out)s\n stderr: %(err)s'),
+            LOG.error('delete: %(vol_id)s failed with '
+                      'stdout: %(out)s\n stderr: %(err)s',
                       {'vol_id': backup.volume_id,
                        'out': out,
                        'err': err})
 
         LOG.debug('Delete %s finished.', backup['id'])
-
-
-def get_backup_driver(context):
-    return TSMBackupDriver(context)
