@@ -28,7 +28,7 @@ from oslo_utils import excutils
 from six import moves
 
 from cinder import exception
-from cinder.i18n import _LE, _LI
+import cinder.privsep.lvm
 from cinder import utils
 
 
@@ -38,6 +38,7 @@ LOG = logging.getLogger(__name__)
 class LVM(executor.Executor):
     """LVM object to enable various LVM related operations."""
     LVM_CMD_PREFIX = ['env', 'LC_ALL=C']
+    _supports_pvs_ignoreskippedcluster = None
 
     def __init__(self, vg_name, root_helper, create_vg=False,
                  physical_volumes=None, lvm_type='default',
@@ -73,11 +74,14 @@ class LVM(executor.Executor):
         self._supports_lvchange_ignoreskipactivation = None
         self.vg_provisioned_capacity = 0.0
 
+        if lvm_type not in ['default', 'thin']:
+            raise exception.Invalid('lvm_type must be "default" or "thin"')
+
         # Ensure LVM_SYSTEM_DIR has been added to LVM.LVM_CMD_PREFIX
         # before the first LVM command is executed, and use the directory
         # where the specified lvm_conf file is located as the value.
 
-        # NOTE(jdg): We use the temp var here becuase LVM_CMD_PREFIX is a
+        # NOTE(jdg): We use the temp var here because LVM_CMD_PREFIX is a
         # class global and if you use append here, you'll literally just keep
         # appending values to the global.
         _lvm_cmd_prefix = ['env', 'LC_ALL=C']
@@ -91,19 +95,17 @@ class LVM(executor.Executor):
         LVM.LVM_CMD_PREFIX = _lvm_cmd_prefix
 
         if create_vg and physical_volumes is not None:
-            self.pv_list = physical_volumes
-
             try:
                 self._create_vg(physical_volumes)
             except putils.ProcessExecutionError as err:
-                LOG.exception(_LE('Error creating Volume Group'))
-                LOG.error(_LE('Cmd     :%s'), err.cmd)
-                LOG.error(_LE('StdOut  :%s'), err.stdout)
-                LOG.error(_LE('StdErr  :%s'), err.stderr)
+                LOG.exception('Error creating Volume Group')
+                LOG.error('Cmd     :%s', err.cmd)
+                LOG.error('StdOut  :%s', err.stdout)
+                LOG.error('StdErr  :%s', err.stderr)
                 raise exception.VolumeGroupCreationFailed(vg_name=self.vg_name)
 
         if self._vg_exists() is False:
-            LOG.error(_LE('Unable to locate Volume Group %s'), vg_name)
+            LOG.error('Unable to locate Volume Group %s', vg_name)
             raise exception.VolumeGroupNotFound(vg_name=vg_name)
 
         # NOTE: we assume that the VG has been activated outside of Cinder
@@ -145,8 +147,26 @@ class LVM(executor.Executor):
         return exists
 
     def _create_vg(self, pv_list):
-        cmd = ['vgcreate', self.vg_name, ','.join(pv_list)]
-        self._execute(*cmd, root_helper=self._root_helper, run_as_root=True)
+        cinder.privsep.lvm.create_volume(self.vg_name, pv_list)
+
+    @utils.retry(retry=utils.retry_if_exit_code, retry_param=139, interval=0.5,
+                 backoff_rate=0.5)
+    def _run_lvm_command(self,
+                         cmd_arg_list: list,
+                         root_helper: str = None,
+                         run_as_root: bool = True) -> tuple:
+        """Run LVM commands with a retry on code 139 to work around LVM bugs.
+
+        Refer to LP bug 1901783, LP bug 1932188.
+        """
+        if not root_helper:
+            root_helper = self._root_helper
+
+        (out, err) = self._execute(*cmd_arg_list,
+                                   root_helper=root_helper,
+                                   run_as_root=run_as_root)
+
+        return (out, err)
 
     def _get_thin_pool_free_space(self, vg_name, thin_pool_name):
         """Returns available thin pool free space.
@@ -167,9 +187,7 @@ class LVM(executor.Executor):
         free_space = 0.0
 
         try:
-            (out, err) = self._execute(*cmd,
-                                       root_helper=self._root_helper,
-                                       run_as_root=True)
+            (out, err) = self._run_lvm_command(cmd)
             if out is not None:
                 out = out.strip()
                 data = out.split(':')
@@ -178,11 +196,14 @@ class LVM(executor.Executor):
                 consumed_space = pool_size / 100 * data_percent
                 free_space = pool_size - consumed_space
                 free_space = round(free_space, 2)
-        except putils.ProcessExecutionError as err:
-            LOG.exception(_LE('Error querying thin pool about data_percent'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+        # Need noqa due to a false error about the 'err' variable being unused
+        # even though it is used in the logging. Possibly related to
+        # https://github.com/PyCQA/pyflakes/issues/378.
+        except putils.ProcessExecutionError as err:  # noqa
+            LOG.exception('Error querying thin pool about data_percent')
+            LOG.error('Cmd     :%s', err.cmd)
+            LOG.error('StdOut  :%s', err.stdout)
+            LOG.error('StdErr  :%s', err.stderr)
 
         return free_space
 
@@ -258,6 +279,24 @@ class LVM(executor.Executor):
         return self._supports_lvchange_ignoreskipactivation
 
     @staticmethod
+    def supports_pvs_ignoreskippedcluster(root_helper):
+        """Property indicating whether pvs supports --ignoreskippedcluster
+
+        Check for LVM version >= 2.02.103.
+        (LVM2 git: baf95bbff cmdline: Add --ignoreskippedcluster.
+        """
+
+        if LVM._supports_pvs_ignoreskippedcluster is not None:
+            return LVM._supports_pvs_ignoreskippedcluster
+
+        LVM._supports_pvs_ignoreskippedcluster = (
+            LVM.get_lvm_version(root_helper) >= (2, 2, 103))
+
+        return LVM._supports_pvs_ignoreskippedcluster
+
+    @staticmethod
+    @utils.retry(retry=utils.retry_if_exit_code, retry_param=139, interval=0.5,
+                 backoff_rate=0.5)  # Bug#1901783
     def get_lv_info(root_helper, vg_name=None, lv_name=None):
         """Retrieve info about LVs (all, in a VG, or a single LV).
 
@@ -283,8 +322,8 @@ class LVM(executor.Executor):
             with excutils.save_and_reraise_exception(reraise=True) as ctx:
                 if "not found" in err.stderr or "Failed to find" in err.stderr:
                     ctx.reraise = False
-                    LOG.info(_LI("Logical Volume not found when querying "
-                                 "LVM info. (vg_name=%(vg)s, lv_name=%(lv)s"),
+                    LOG.info("Logical Volume not found when querying "
+                             "LVM info. (vg_name=%(vg)s, lv_name=%(lv)s",
                              {'vg': vg_name, 'lv': lv_name})
                     out = None
 
@@ -334,6 +373,9 @@ class LVM(executor.Executor):
                                     '-o', 'vg_name,name,size,free',
                                     '--separator', field_sep,
                                     '--nosuffix']
+        if LVM.supports_pvs_ignoreskippedcluster(root_helper):
+            cmd.append('--ignoreskippedcluster')
+
         (out, _err) = putils.execute(*cmd,
                                      root_helper=root_helper,
                                      run_as_root=True)
@@ -396,7 +438,7 @@ class LVM(executor.Executor):
         vg_list = self.get_all_volume_groups(self._root_helper, self.vg_name)
 
         if len(vg_list) != 1:
-            LOG.error(_LE('Unable to find VG: %s'), self.vg_name)
+            LOG.error('Unable to find VG: %s', self.vg_name)
             raise exception.VolumeGroupNotFound(vg_name=self.vg_name)
 
         self.vg_size = float(vg_list[0]['size'])
@@ -439,7 +481,7 @@ class LVM(executor.Executor):
                 if not lv['size'][-1].isdigit():
                     lvsize = lvsize[:-1]
                 if lv['name'] == self.vg_thin_pool:
-                    self.vg_thin_pool_size = lvsize
+                    self.vg_thin_pool_size = float(lvsize)
                     tpfs = self._get_thin_pool_free_space(self.vg_name,
                                                           self.vg_thin_pool)
                     self.vg_thin_pool_free_space = tpfs
@@ -483,9 +525,9 @@ class LVM(executor.Executor):
         """
 
         if not self.supports_thin_provisioning(self._root_helper):
-            LOG.error(_LE('Requested to setup thin provisioning, '
-                          'however current LVM version does not '
-                          'support it.'))
+            LOG.error('Requested to setup thin provisioning, '
+                      'however current LVM version does not '
+                      'support it.')
             return None
 
         if name is None:
@@ -503,9 +545,7 @@ class LVM(executor.Executor):
                                       'size': size_str,
                                       'free': self.vg_free_space})
 
-        self._execute(*cmd,
-                      root_helper=self._root_helper,
-                      run_as_root=True)
+        self._run_lvm_command(cmd)
 
         self.vg_thin_pool = name
         return size_str
@@ -529,7 +569,7 @@ class LVM(executor.Executor):
                                         '-L', size_str]
 
         if mirror_count > 0:
-            cmd.extend(['-m', mirror_count, '--nosync',
+            cmd.extend(['--type=mirror', '-m', mirror_count, '--nosync',
                         '--mirrorlog', 'mirrored'])
             terras = int(size_str[:-1]) / 1024.0
             if terras >= 1.5:
@@ -539,14 +579,14 @@ class LVM(executor.Executor):
                 cmd.extend(['-R', str(rsize)])
 
         try:
-            self._execute(*cmd,
-                          root_helper=self._root_helper,
-                          run_as_root=True)
+            self._run_lvm_command(cmd)
         except putils.ProcessExecutionError as err:
-            LOG.exception(_LE('Error creating Volume'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+            LOG.exception('Error creating Volume')
+            LOG.error('Cmd     :%s', err.cmd)
+            LOG.error('StdOut  :%s', err.stdout)
+            LOG.error('StdErr  :%s', err.stderr)
+            LOG.error('Current state: %s',
+                      self.get_all_volume_groups(self._root_helper))
             raise
 
     @utils.retry(putils.ProcessExecutionError)
@@ -560,7 +600,7 @@ class LVM(executor.Executor):
         """
         source_lvref = self.get_volume(source_lv_name)
         if source_lvref is None:
-            LOG.error(_LE("Trying to create snapshot by non-existent LV: %s"),
+            LOG.error("Trying to create snapshot by non-existent LV: %s",
                       source_lv_name)
             raise exception.VolumeDeviceNotFound(device=source_lv_name)
         cmd = LVM.LVM_CMD_PREFIX + ['lvcreate', '--name', name, '--snapshot',
@@ -570,14 +610,12 @@ class LVM(executor.Executor):
             cmd.extend(['-L', '%sg' % (size)])
 
         try:
-            self._execute(*cmd,
-                          root_helper=self._root_helper,
-                          run_as_root=True)
+            self._run_lvm_command(cmd)
         except putils.ProcessExecutionError as err:
-            LOG.exception(_LE('Error creating snapshot'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+            LOG.exception('Error creating snapshot')
+            LOG.error('Cmd     :%s', err.cmd)
+            LOG.error('StdOut  :%s', err.stdout)
+            LOG.error('StdErr  :%s', err.stderr)
             raise
 
     def _mangle_lv_name(self, name):
@@ -590,15 +628,14 @@ class LVM(executor.Executor):
     def _lv_is_active(self, name):
         cmd = LVM.LVM_CMD_PREFIX + ['lvdisplay', '--noheading', '-C', '-o',
                                     'Attr', '%s/%s' % (self.vg_name, name)]
-        out, _err = self._execute(*cmd,
-                                  root_helper=self._root_helper,
-                                  run_as_root=True)
+        out, _err = self._run_lvm_command(cmd)
         if out:
             out = out.strip()
             if (out[4] == 'a'):
                 return True
         return False
 
+    @utils.retry(exception.VolumeNotDeactivated, retries=1, interval=2)
     def deactivate_lv(self, name):
         lv_path = self.vg_name + '/' + self._mangle_lv_name(name)
         cmd = ['lvchange', '-a', 'n']
@@ -608,18 +645,18 @@ class LVM(executor.Executor):
                           root_helper=self._root_helper,
                           run_as_root=True)
         except putils.ProcessExecutionError as err:
-            LOG.exception(_LE('Error deactivating LV'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+            LOG.exception('Error deactivating LV, retry may be possible')
+            LOG.error('Cmd     :%s', err.cmd)
+            LOG.error('StdOut  :%s', err.stdout)
+            LOG.error('StdErr  :%s', err.stderr)
             raise
 
         # Wait until lv is deactivated to return in
         # order to prevent a race condition.
         self._wait_for_volume_deactivation(name)
 
-    @utils.retry(exceptions=exception.VolumeNotDeactivated, retries=3,
-                 backoff_rate=1)
+    @utils.retry(retry_param=exception.VolumeNotDeactivated, retries=5,
+                 backoff_rate=2)
     def _wait_for_volume_deactivation(self, name):
         LOG.debug("Checking to see if volume %s has been deactivated.",
                   name)
@@ -629,13 +666,14 @@ class LVM(executor.Executor):
         else:
             LOG.debug("Volume %s has been deactivated.", name)
 
+    @utils.retry(putils.ProcessExecutionError, retries=5, backoff_rate=2)
     def activate_lv(self, name, is_snapshot=False, permanent=False):
         """Ensure that logical volume/snapshot logical volume is activated.
 
         :param name: Name of LV to activate
         :param is_snapshot: whether LV is a snapshot
         :param permanent: whether we should drop skipactivation flag
-        :raises: putils.ProcessExecutionError
+        :raises putils.ProcessExecutionError:
         """
 
         # This is a no-op if requested for a snapshot on a version
@@ -652,11 +690,12 @@ class LVM(executor.Executor):
         cmd = ['lvchange', '-a', 'y', '--yes']
 
         if self.supports_lvchange_ignoreskipactivation:
-            cmd.append('-K')
             # If permanent=True is specified, drop the skipactivation flag in
             # order to make this LV automatically activated after next reboot.
             if permanent:
                 cmd += ['-k', 'n']
+            else:
+                cmd.append('-K')
 
         cmd.append(lv_path)
 
@@ -665,10 +704,10 @@ class LVM(executor.Executor):
                           root_helper=self._root_helper,
                           run_as_root=True)
         except putils.ProcessExecutionError as err:
-            LOG.exception(_LE('Error activating LV'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+            LOG.exception('Error activating LV')
+            LOG.error('Cmd     :%s', err.cmd)
+            LOG.error('StdOut  :%s', err.stdout)
+            LOG.error('StdErr  :%s', err.stderr)
             raise
 
     @utils.retry(putils.ProcessExecutionError)
@@ -680,9 +719,7 @@ class LVM(executor.Executor):
         """
 
         def run_udevadm_settle():
-            self._execute('udevadm', 'settle',
-                          root_helper=self._root_helper, run_as_root=True,
-                          check_exit_code=False)
+            cinder.privsep.lvm.udevadm_settle()
 
         # LV removal seems to be a race with other writers or udev in
         # some cases (see LP #1270192), so we enable retry deactivation
@@ -719,21 +756,25 @@ class LVM(executor.Executor):
                       'udev settle.', name)
 
     def revert(self, snapshot_name):
-        """Revert an LV from snapshot.
+        """Revert an LV to snapshot.
 
         :param snapshot_name: Name of snapshot to revert
-
         """
-        self._execute('lvconvert', '--merge',
-                      snapshot_name, root_helper=self._root_helper,
-                      run_as_root=True)
+
+        try:
+            cinder.privsep.lvm.lvconvert(self.vg_name, snapshot_name)
+        except putils.ProcessExecutionError as err:
+            LOG.exception('Error Revert Volume')
+            LOG.error('Cmd     :%s', err.cmd)
+            LOG.error('StdOut  :%s', err.stdout)
+            LOG.error('StdErr  :%s', err.stderr)
+            raise
 
     def lv_has_snapshot(self, name):
         cmd = LVM.LVM_CMD_PREFIX + ['lvdisplay', '--noheading', '-C', '-o',
-                                    'Attr', '%s/%s' % (self.vg_name, name)]
-        out, _err = self._execute(*cmd,
-                                  root_helper=self._root_helper,
-                                  run_as_root=True)
+                                    'Attr', '--readonly',
+                                    '%s/%s' % (self.vg_name, name)]
+        out, _err = self._run_lvm_command(cmd)
         if out:
             out = out.strip()
             if (out[0] == 'o') or (out[0] == 'O'):
@@ -744,9 +785,7 @@ class LVM(executor.Executor):
         """Return True if LV is a snapshot, False otherwise."""
         cmd = LVM.LVM_CMD_PREFIX + ['lvdisplay', '--noheading', '-C', '-o',
                                     'Attr', '%s/%s' % (self.vg_name, name)]
-        out, _err = self._execute(*cmd,
-                                  root_helper=self._root_helper,
-                                  run_as_root=True)
+        out, _err = self._run_lvm_command(cmd)
         out = out.strip()
         if out:
             if (out[0] == 's'):
@@ -757,9 +796,7 @@ class LVM(executor.Executor):
         """Return True if LV is currently open, False otherwise."""
         cmd = LVM.LVM_CMD_PREFIX + ['lvdisplay', '--noheading', '-C', '-o',
                                     'Attr', '%s/%s' % (self.vg_name, name)]
-        out, _err = self._execute(*cmd,
-                                  root_helper=self._root_helper,
-                                  run_as_root=True)
+        out, _err = self._run_lvm_command(cmd)
         out = out.strip()
         if out:
             if (out[5] == 'o'):
@@ -770,9 +807,7 @@ class LVM(executor.Executor):
         """Return the origin of an LV that is a snapshot, None otherwise."""
         cmd = LVM.LVM_CMD_PREFIX + ['lvdisplay', '--noheading', '-C', '-o',
                                     'Origin', '%s/%s' % (self.vg_name, name)]
-        out, _err = self._execute(*cmd,
-                                  root_helper=self._root_helper,
-                                  run_as_root=True)
+        out, _err = self._run_lvm_command(cmd)
         out = out.strip()
         if out:
             return out
@@ -784,19 +819,21 @@ class LVM(executor.Executor):
         # deactivated, but Thin Volumes with snaps have attribute 'V'
         # and won't be deactivated because the lv_has_snapshot method looks
         # for 'o' or 'O'
-        if self.lv_has_snapshot(lv_name):
+        has_snapshot = self.lv_has_snapshot(lv_name)
+        if has_snapshot:
             self.deactivate_lv(lv_name)
         try:
             cmd = LVM.LVM_CMD_PREFIX + ['lvextend', '-L', new_size,
                                         '%s/%s' % (self.vg_name, lv_name)]
-            self._execute(*cmd, root_helper=self._root_helper,
-                          run_as_root=True)
+            self._run_lvm_command(cmd)
         except putils.ProcessExecutionError as err:
-            LOG.exception(_LE('Error extending Volume'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+            LOG.exception('Error extending Volume')
+            LOG.error('Cmd     :%s', err.cmd)
+            LOG.error('StdOut  :%s', err.stdout)
+            LOG.error('StdErr  :%s', err.stderr)
             raise
+        if has_snapshot:
+            self.activate_lv(lv_name)
 
     def vg_mirror_free_space(self, mirror_count):
         free_capacity = 0.0
@@ -826,12 +863,10 @@ class LVM(executor.Executor):
         """Change the name of an existing volume."""
 
         try:
-            self._execute('lvrename', self.vg_name, lv_name, new_name,
-                          root_helper=self._root_helper,
-                          run_as_root=True)
+            cinder.privsep.lvm.lvrename(self.vg_name, lv_name, new_name)
         except putils.ProcessExecutionError as err:
-            LOG.exception(_LE('Error renaming logical volume'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+            LOG.exception('Error renaming logical volume')
+            LOG.error('Cmd     :%s', err.cmd)
+            LOG.error('StdOut  :%s', err.stdout)
+            LOG.error('StdErr  :%s', err.stderr)
             raise
