@@ -645,7 +645,7 @@ def fetch_to_volume_format(context, image_service,
     # large and cause disk full errors which would confuse users.
     # Unfortunately it seems that you can't pipe to 'qemu-img convert' because
     # it seeks. Maybe we can think of something for a future version.
-    with temporary_file() as tmp:
+    with temporary_file(prefix='image_download_%s_' % image_id) as tmp:
         has_meta = False if not image_meta else True
         try:
             format_raw = True if image_meta['disk_format'] == 'raw' else False
@@ -763,7 +763,7 @@ def upload_volume(context, image_service, image_meta, volume_path,
                                              base_image_ref=base_image_ref)
             return
 
-    with temporary_file() as tmp:
+    with temporary_file(prefix='vol_upload_') as tmp:
         LOG.debug("%s was %s, converting to %s",
                   image_id, volume_format, image_meta['disk_format'])
 
@@ -997,7 +997,8 @@ class TemporaryImages(object):
     @contextlib.contextmanager
     def fetch(cls, image_service, context, image_id, suffix=''):
         tmp_images = cls.for_image_service(image_service).temporary_images
-        with temporary_file(suffix=suffix) as tmp:
+        with temporary_file(prefix='image_fetch_%s_' % image_id,
+                            suffix=suffix) as tmp:
             fetch_verify_image(context, image_service, image_id, tmp)
             user = context.user_id
             if not tmp_images.get(user):
