@@ -14,6 +14,11 @@
 
 """Cinder common internal object model"""
 
+try:
+    from collections.abc import Callable
+except ImportError:
+    from collections import Callable
+
 import contextlib
 import datetime
 
@@ -145,6 +150,7 @@ OBJ_VERSIONS.add('1.34', {'VolumeAttachment': '1.3'})
 OBJ_VERSIONS.add('1.35', {'Backup': '1.6', 'BackupImport': '1.6'})
 OBJ_VERSIONS.add('1.36', {'RequestSpec': '1.4'})
 OBJ_VERSIONS.add('1.37', {'RequestSpec': '1.5'})
+OBJ_VERSIONS.add('1.38', {'Backup': '1.7', 'BackupImport': '1.7'})
 
 
 class CinderObjectRegistry(base.VersionedObjectRegistry):
@@ -160,7 +166,8 @@ class CinderObjectRegistry(base.VersionedObjectRegistry):
         setattr(objects, cls.obj_name(), cls)
 
         # If registering class has a callable initialization method, call it.
-        if callable(getattr(cls, 'cinder_ovo_cls_init', None)):
+        if isinstance(getattr(cls, 'cinder_ovo_cls_init', None),
+                      Callable):
             cls.cinder_ovo_cls_init()
 
 
@@ -276,6 +283,7 @@ class CinderPersistentObject(object):
         except (ImportError, AttributeError):
             msg = _("Couldn't find ORM model for Persistent Versioned "
                     "Object %s.") % cls.obj_name()
+            LOG.exception("Failed to initialize object.")
             raise exception.ProgrammingError(reason=msg)
 
     @contextlib.contextmanager
@@ -526,6 +534,9 @@ class ClusteredObject(object):
                     'storage backends.')
             raise exception.InvalidInput(reason=msg)
 
+    # The object's resource backend depends on whether it's clustered.
+    resource_backend = service_topic_queue
+
 
 class CinderObjectSerializer(base.VersionedObjectSerializer):
     OBJ_BASE_CLASS = CinderObject
@@ -567,7 +578,7 @@ class CinderObjectSerializer(base.VersionedObjectSerializer):
             entity = self._process_iterable(context, self.serialize_entity,
                                             entity)
         elif (hasattr(entity, 'obj_to_primitive') and
-              callable(entity.obj_to_primitive)):
+              isinstance(entity.obj_to_primitive, Callable)):
             # NOTE(dulek): Backport outgoing object to the capped version.
             backport_ver = self._get_capped_obj_version(entity)
             entity = entity.obj_to_primitive(backport_ver, self.manifest)
