@@ -21,7 +21,9 @@ from cinder import exception
 from cinder.i18n import _
 from cinder import objects
 from cinder.objects import base
-
+from cinder.objects import cleanable
+from cinder.objects import fields as c_fields
+from cinder.volume import volume_types
 
 CONF = cfg.CONF
 
@@ -48,7 +50,7 @@ class MetadataObject(dict):
 
 
 @base.CinderObjectRegistry.register
-class Volume(base.CinderPersistentObject, base.CinderObject,
+class Volume(cleanable.CinderCleanableObject, base.CinderObject,
              base.CinderObjectDictCompat, base.CinderComparableObject,
              base.ClusteredObject):
     # Version 1.0: Initial version
@@ -58,7 +60,10 @@ class Volume(base.CinderPersistentObject, base.CinderObject,
     # Version 1.3: Added finish_volume_migration()
     # Version 1.4: Added cluster fields
     # Version 1.5: Added group
-    VERSION = '1.5'
+    # Version 1.6: This object is now cleanable (adds rows to workers table)
+    # Version 1.7: Added service_uuid
+    # Version 1.8: Added shared_targets
+    VERSION = '1.8'
 
     OPTIONAL_FIELDS = ('metadata', 'admin_metadata', 'glance_metadata',
                        'volume_type', 'volume_attachment', 'consistencygroup',
@@ -80,7 +85,7 @@ class Volume(base.CinderPersistentObject, base.CinderObject,
         'size': fields.IntegerField(nullable=True),
         'availability_zone': fields.StringField(nullable=True),
         'status': fields.StringField(nullable=True),
-        'attach_status': fields.StringField(nullable=True),
+        'attach_status': c_fields.VolumeAttachStatusField(nullable=True),
         'migration_status': fields.StringField(nullable=True),
 
         'scheduled_at': fields.DateTimeField(nullable=True),
@@ -122,6 +127,8 @@ class Volume(base.CinderPersistentObject, base.CinderObject,
                                                nullable=True),
         'snapshots': fields.ObjectField('SnapshotList', nullable=True),
         'group': fields.ObjectField('Group', nullable=True),
+        'service_uuid': fields.StringField(nullable=True),
+        'shared_targets': fields.BooleanField(default=True, nullable=True),
     }
 
     # NOTE(thangp): obj_extra_fields is used to hold properties that are not
@@ -131,7 +138,8 @@ class Volume(base.CinderPersistentObject, base.CinderObject,
 
     @classmethod
     def _get_expected_attrs(cls, context, *args, **kwargs):
-        expected_attrs = ['metadata', 'volume_type', 'volume_type.extra_specs']
+        expected_attrs = ['metadata', 'volume_type', 'volume_type.extra_specs',
+                          'volume_attachment']
         if context.is_admin:
             expected_attrs.append('admin_metadata')
 
@@ -173,6 +181,13 @@ class Volume(base.CinderPersistentObject, base.CinderObject,
         md = {d['key']: d['value'] for d in value}
         self.admin_metadata = md
 
+    def admin_metadata_update(self, metadata, delete, add=True, update=True):
+        new_metadata = db.volume_admin_metadata_update(self._context, self.id,
+                                                       metadata, delete, add,
+                                                       update)
+        self.admin_metadata = new_metadata
+        self._reset_metadata_tracking(fields=('admin_metadata',))
+
     @property
     def volume_glance_metadata(self):
         md = [MetadataObject(k, v) for k, v in self.glance_metadata.items()]
@@ -185,9 +200,6 @@ class Volume(base.CinderPersistentObject, base.CinderObject,
 
     def __init__(self, *args, **kwargs):
         super(Volume, self).__init__(*args, **kwargs)
-        self._orig_metadata = {}
-        self._orig_admin_metadata = {}
-        self._orig_glance_metadata = {}
 
         self._reset_metadata_tracking()
 
@@ -230,14 +242,18 @@ class Volume(base.CinderPersistentObject, base.CinderObject,
 
     def obj_make_compatible(self, primitive, target_version):
         """Make a Volume representation compatible with a target version."""
+        added_fields = (((1, 4), ('cluster', 'cluster_name')),
+                        ((1, 5), ('group', 'group_id')),
+                        ((1, 7), ('service_uuid')))
+
         # Convert all related objects
         super(Volume, self).obj_make_compatible(primitive, target_version)
 
         target_version = versionutils.convert_version_to_tuple(target_version)
-        # Before v1.4 we didn't have cluster fields so we have to remove them.
-        if target_version < (1, 4):
-            for obj_field in ('cluster', 'cluster_name'):
-                primitive.pop(obj_field, None)
+        for version, remove_fields in added_fields:
+            if target_version < version:
+                for obj_field in remove_fields:
+                    primitive.pop(obj_field, None)
 
     @classmethod
     def _from_db_object(cls, context, volume, db_volume, expected_attrs=None):
@@ -279,7 +295,7 @@ class Volume(base.CinderPersistentObject, base.CinderObject,
                 objects.VolumeAttachment,
                 db_volume.get('volume_attachment'))
             volume.volume_attachment = attachments
-        if 'consistencygroup' in expected_attrs:
+        if volume.consistencygroup_id and 'consistencygroup' in expected_attrs:
             consistencygroup = objects.ConsistencyGroup(context)
             consistencygroup._from_db_object(context,
                                              consistencygroup,
@@ -301,7 +317,7 @@ class Volume(base.CinderPersistentObject, base.CinderObject,
                                                 db_cluster)
             else:
                 volume.cluster = None
-        if 'group' in expected_attrs:
+        if volume.group_id and 'group' in expected_attrs:
             group = objects.Group(context)
             group._from_db_object(context,
                                   group,
@@ -330,14 +346,26 @@ class Volume(base.CinderPersistentObject, base.CinderObject,
         if 'group' in updates:
             raise exception.ObjectActionError(
                 action='create', reason=_('group assigned'))
+        if ('volume_type_id' not in updates or
+                updates['volume_type_id'] is None):
+            updates['volume_type_id'] = (
+                volume_types.get_default_volume_type()['id'])
 
         db_volume = db.volume_create(self._context, updates)
-        self._from_db_object(self._context, self, db_volume)
+        expected_attrs = self._get_expected_attrs(self._context)
+        self._from_db_object(self._context, self, db_volume, expected_attrs)
 
     def save(self):
         updates = self.cinder_obj_get_changes()
         if updates:
-            if 'consistencygroup' in updates:
+            # NOTE(xyang): Allow this to pass if 'consistencygroup' is
+            # set to None. This is to support backward compatibility.
+            # Also remove 'consistencygroup' from updates because
+            # consistencygroup is the name of a relationship in the ORM
+            # Volume model, so SQLA tries to do some kind of update of
+            # the foreign key based on the provided updates if
+            # 'consistencygroup' is in updates.
+            if updates.pop('consistencygroup', None):
                 raise exception.ObjectActionError(
                     action='save', reason=_('consistencygroup changed'))
             if 'group' in updates:
@@ -364,7 +392,17 @@ class Volume(base.CinderPersistentObject, base.CinderObject,
                 self.admin_metadata = db.volume_admin_metadata_update(
                     self._context, self.id, metadata, True)
 
-            db.volume_update(self._context, self.id, updates)
+            # When we are creating a volume and we change from 'creating'
+            # status to 'downloading' status we have to change the worker entry
+            # in the DB to reflect this change, otherwise the cleanup will
+            # not be performed as it will be mistaken for a volume that has
+            # been somehow changed (reset status, forced operation...)
+            if updates.get('status') == 'downloading':
+                self.set_worker()
+
+            # updates are changed after popping out metadata.
+            if updates:
+                db.volume_update(self._context, self.id, updates)
             self.obj_reset_changes()
 
     def destroy(self):
@@ -411,9 +449,12 @@ class Volume(base.CinderPersistentObject, base.CinderObject,
                 self._context, self.id)
             self.volume_attachment = attachments
         elif attrname == 'consistencygroup':
-            consistencygroup = objects.ConsistencyGroup.get_by_id(
-                self._context, self.consistencygroup_id)
-            self.consistencygroup = consistencygroup
+            if self.consistencygroup_id is None:
+                self.consistencygroup = None
+            else:
+                consistencygroup = objects.ConsistencyGroup.get_by_id(
+                    self._context, self.consistencygroup_id)
+                self.consistencygroup = consistencygroup
         elif attrname == 'snapshots':
             self.snapshots = objects.SnapshotList.get_all_for_volume(
                 self._context, self.id)
@@ -426,9 +467,12 @@ class Volume(base.CinderPersistentObject, base.CinderObject,
             else:
                 self.cluster = None
         elif attrname == 'group':
-            group = objects.Group.get_by_id(
-                self._context, self.group_id)
-            self.group = group
+            if self.group_id is None:
+                self.group = None
+            else:
+                group = objects.Group.get_by_id(
+                    self._context, self.group_id)
+                self.group = group
 
         self.obj_reset_changes(fields=[attrname])
 
@@ -447,7 +491,8 @@ class Volume(base.CinderPersistentObject, base.CinderObject,
         # end of migration because we want to keep the original volume id
         # in the DB but now pointing to the migrated volume.
         skip = ({'id', 'provider_location', 'glance_metadata',
-                 'volume_type'} | set(self.obj_extra_fields))
+                 'volume_type', 'volume_attachment'}
+                | set(self.obj_extra_fields))
         for key in set(dest_volume.fields.keys()) - skip:
             # Only swap attributes that are already set.  We do not want to
             # unexpectedly trigger a lazy-load.
@@ -486,6 +531,57 @@ class Volume(base.CinderPersistentObject, base.CinderObject,
         dest_volume.save()
         return dest_volume
 
+    def get_latest_snapshot(self):
+        """Get volume's latest snapshot"""
+        snapshot_db = db.snapshot_get_latest_for_volume(self._context, self.id)
+        snapshot = objects.Snapshot(self._context)
+        return snapshot._from_db_object(self._context,
+                                        snapshot, snapshot_db)
+
+    @staticmethod
+    def _is_cleanable(status, obj_version):
+        # Before 1.6 we didn't have workers table, so cleanup wasn't supported.
+        # cleaning.
+        if obj_version and obj_version < 1.6:
+            return False
+        return status in ('creating', 'deleting', 'uploading', 'downloading')
+
+    def begin_attach(self, attach_mode):
+        attachment = objects.VolumeAttachment(
+            context=self._context,
+            attach_status=c_fields.VolumeAttachStatus.ATTACHING,
+            volume_id=self.id)
+        attachment.create()
+        with self.obj_as_admin():
+            self.admin_metadata['attached_mode'] = attach_mode
+            self.save()
+        return attachment
+
+    def finish_detach(self, attachment_id):
+        with self.obj_as_admin():
+            volume_updates, attachment_updates = (
+                db.volume_detached(self._context, self.id, attachment_id))
+            db.volume_admin_metadata_delete(self._context, self.id,
+                                            'attached_mode')
+            self.admin_metadata.pop('attached_mode', None)
+        # Remove attachment in volume only when this field is loaded.
+        if attachment_updates and self.obj_attr_is_set('volume_attachment'):
+            for i, attachment in enumerate(self.volume_attachment):
+                if attachment.id == attachment_id:
+                    del self.volume_attachment.objects[i]
+                    break
+
+        self.update(volume_updates)
+        self.obj_reset_changes(
+            list(volume_updates.keys()) +
+            ['volume_attachment', 'admin_metadata'])
+
+    def is_replicated(self):
+        return self.volume_type and self.volume_type.is_replicated()
+
+    def is_multiattach(self):
+        return self.volume_type and self.volume_type.is_multiattach()
+
 
 @base.CinderObjectRegistry.register
 class VolumeList(base.ObjectListBase, base.CinderObject):
@@ -516,15 +612,15 @@ class VolumeList(base.ObjectListBase, base.CinderObject):
 
     @classmethod
     def _get_expected_attrs(cls, context, *args, **kwargs):
-        expected_attrs = ['metadata', 'volume_type']
+        expected_attrs = ['metadata', 'volume_type', 'volume_attachment']
         if context.is_admin:
             expected_attrs.append('admin_metadata')
 
         return expected_attrs
 
     @classmethod
-    def get_all(cls, context, marker, limit, sort_keys=None, sort_dirs=None,
-                filters=None, offset=None):
+    def get_all(cls, context, marker=None, limit=None, sort_keys=None,
+                sort_dirs=None, filters=None, offset=None):
         volumes = db.volume_get_all(context, marker, limit,
                                     sort_keys=sort_keys, sort_dirs=sort_dirs,
                                     filters=filters, offset=offset)
@@ -557,7 +653,7 @@ class VolumeList(base.ObjectListBase, base.CinderObject):
                                   volumes, expected_attrs=expected_attrs)
 
     @classmethod
-    def get_all_by_project(cls, context, project_id, marker, limit,
+    def get_all_by_project(cls, context, project_id, marker=None, limit=None,
                            sort_keys=None, sort_dirs=None, filters=None,
                            offset=None):
         volumes = db.volume_get_all_by_project(context, project_id, marker,
@@ -569,11 +665,13 @@ class VolumeList(base.ObjectListBase, base.CinderObject):
                                   volumes, expected_attrs=expected_attrs)
 
     @classmethod
-    def get_volume_summary_all(cls, context):
-        volumes = db.get_volume_summary_all(context)
+    def get_volume_summary(cls, context, project_only, filters=None):
+        volumes = db.get_volume_summary(context, project_only, filters)
         return volumes
 
     @classmethod
-    def get_volume_summary_by_project(cls, context, project_id):
-        volumes = db.get_volume_summary_by_project(context, project_id)
-        return volumes
+    def get_all_active_by_window(cls, context, begin, end):
+        volumes = db.volume_get_all_active_by_window(context, begin, end)
+        expected_attrs = cls._get_expected_attrs(context)
+        return base.obj_make_list(context, cls(context), objects.Volume,
+                                  volumes, expected_attrs=expected_attrs)
