@@ -13,8 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
 
-import mock
 from oslo_serialization import jsonutils
 from six.moves import http_client
 import webob
@@ -22,11 +22,11 @@ import webob
 from cinder import context
 from cinder.objects import fields
 from cinder.policies import snapshots as snap_policy
-from cinder import test
 from cinder.tests.unit.api import fakes
 from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import fake_snapshot
 from cinder.tests.unit import fake_volume
+from cinder.tests.unit import test
 
 
 UUID1 = fake.SNAPSHOT_ID
