@@ -14,8 +14,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+
 import ddt
-import mock
 from six.moves import http_client
 import webob
 
@@ -24,9 +25,9 @@ from cinder import context
 from cinder import db
 from cinder import exception
 from cinder import objects
-from cinder import test
 from cinder.tests.unit.api import fakes
 from cinder.tests.unit import fake_constants as fake
+from cinder.tests.unit import test
 
 
 def stub_qos_specs(id):
@@ -480,7 +481,7 @@ class QoSSpecManageApiTest(test.TestCase):
     @ddt.data(({'name': None}, exception.ValidationError),
               ({'name': ''}, exception.ValidationError),
               ({'name': '  '}, exception.ValidationError),
-              ({'name': 'n' * 256}, exception.InvalidInput))
+              ({'name': 'n' * 256}, exception.ValidationError))
     @ddt.unpack
     def test_create_qos_with_invalid_spec_name(self, value, exception_class):
         body = {'qos_specs': value}
