@@ -13,7 +13,8 @@
 #   License for the specific language governing permissions and limitations
 #   under the License.
 
-import mock
+from unittest import mock
+
 from oslo_config import cfg
 import oslo_messaging as messaging
 from oslo_serialization import jsonutils
@@ -25,10 +26,10 @@ from cinder.common import constants
 from cinder import context
 from cinder import exception
 from cinder import objects
-from cinder import test
 from cinder.tests.unit.api import fakes
 from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import fake_service
+from cinder.tests.unit import test
 
 CONF = cfg.CONF
 
