@@ -13,11 +13,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""
-Tests for volume transfer code.
-"""
+"""Tests for volume transfer code."""
+
+from unittest import mock
 
-import mock
 from oslo_serialization import jsonutils
 from six.moves import http_client
 import webob
@@ -28,10 +27,10 @@ from cinder import db
 from cinder import exception
 from cinder.objects import fields
 from cinder import quota
-from cinder import test
 from cinder.tests.unit.api import fakes
 from cinder.tests.unit.api.v2 import fakes as v2_fakes
 from cinder.tests.unit import fake_constants as fake
+from cinder.tests.unit import test
 import cinder.transfer
 
 
