@@ -13,8 +13,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+import datetime
+from unittest import mock
+
+import ddt
 from oslo_config import cfg
+import pytz
+from six.moves import http_client
 from six.moves.urllib import parse as urllib
 import webob
 
@@ -25,12 +30,13 @@ from cinder import db
 from cinder import exception
 from cinder import objects
 from cinder.objects import fields
-from cinder import test
+from cinder.scheduler import rpcapi as scheduler_rpcapi
 from cinder.tests.unit.api import fakes
-from cinder.tests.unit.api.v2 import stubs
+from cinder.tests.unit.api.v2 import fakes as v2_fakes
 from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import fake_snapshot
 from cinder.tests.unit import fake_volume
+from cinder.tests.unit import test
 from cinder.tests.unit import utils
 from cinder import volume
 
@@ -58,12 +64,12 @@ def _get_default_snapshot_param():
     }
 
 
-def stub_snapshot_delete(self, context, snapshot):
+def fake_snapshot_delete(self, context, snapshot):
     if snapshot['id'] != UUID:
         raise exception.SnapshotNotFound(snapshot['id'])
 
 
-def stub_snapshot_get(self, context, snapshot_id):
+def fake_snapshot_get(self, context, snapshot_id):
     if snapshot_id != UUID:
         raise exception.SnapshotNotFound(snapshot_id)
 
@@ -71,27 +77,21 @@ def stub_snapshot_get(self, context, snapshot_id):
     return param
 
 
-def stub_snapshot_get_all(self, context, search_opts=None):
+def fake_snapshot_get_all(self, context, search_opts=None):
     param = _get_default_snapshot_param()
     return [param]
 
 
+@ddt.ddt
 class SnapshotApiTest(test.TestCase):
     def setUp(self):
         super(SnapshotApiTest, self).setUp()
+        self.mock_object(scheduler_rpcapi.SchedulerAPI, 'create_snapshot')
         self.controller = snapshots.SnapshotsController()
-
-        self.stubs.Set(db, 'snapshot_get_all_by_project',
-                       stubs.stub_snapshot_get_all_by_project)
-        self.stubs.Set(db, 'snapshot_get_all',
-                       stubs.stub_snapshot_get_all)
-
         self.ctx = context.RequestContext(fake.USER_ID, fake.PROJECT_ID, True)
 
-    @mock.patch(
-        'cinder.api.openstack.wsgi.Controller.validate_name_and_description')
-    def test_snapshot_create(self, mock_validate):
-        volume = utils.create_volume(self.ctx)
+    def test_snapshot_create(self):
+        volume = utils.create_volume(self.ctx, volume_type_id=None)
         snapshot_name = 'Snapshot Test Name'
         snapshot_description = 'Snapshot Test Desc'
         snapshot = {
@@ -103,29 +103,49 @@ class SnapshotApiTest(test.TestCase):
 
         body = dict(snapshot=snapshot)
         req = fakes.HTTPRequest.blank('/v2/snapshots')
-        resp_dict = self.controller.create(req, body)
+        resp_dict = self.controller.create(req, body=body)
 
         self.assertIn('snapshot', resp_dict)
         self.assertEqual(snapshot_name, resp_dict['snapshot']['name'])
         self.assertEqual(snapshot_description,
                          resp_dict['snapshot']['description'])
-        self.assertTrue(mock_validate.called)
         self.assertIn('updated_at', resp_dict['snapshot'])
         db.volume_destroy(self.ctx, volume.id)
 
-    def test_snapshot_create_force(self):
-        volume = utils.create_volume(self.ctx, status='in-use')
+    def test_snapshot_create_with_null_validate(self):
+
+        volume = utils.create_volume(self.ctx, volume_type_id=None)
+        snapshot = {
+            "volume_id": volume.id,
+            "force": False,
+            "name": None,
+            "description": None
+        }
+
+        body = dict(snapshot=snapshot)
+        req = fakes.HTTPRequest.blank('/v2/snapshots')
+        resp_dict = self.controller.create(req, body=body)
+
+        self.assertIn('snapshot', resp_dict)
+        self.assertIsNone(resp_dict['snapshot']['name'])
+        self.assertIsNone(resp_dict['snapshot']['description'])
+        db.volume_destroy(self.ctx, volume.id)
+
+    @ddt.data(True, 'y', 'true', 'yes', '1', 'on')
+    def test_snapshot_create_force(self, force_param):
+        volume = utils.create_volume(self.ctx, status='in-use',
+                                     volume_type_id=None)
         snapshot_name = 'Snapshot Test Name'
         snapshot_description = 'Snapshot Test Desc'
         snapshot = {
             "volume_id": volume.id,
-            "force": True,
+            "force": force_param,
             "name": snapshot_name,
             "description": snapshot_description
         }
         body = dict(snapshot=snapshot)
         req = fakes.HTTPRequest.blank('/v2/snapshots')
-        resp_dict = self.controller.create(req, body)
+        resp_dict = self.controller.create(req, body=body)
 
         self.assertIn('snapshot', resp_dict)
         self.assertEqual(snapshot_name,
@@ -134,18 +154,49 @@ class SnapshotApiTest(test.TestCase):
                          resp_dict['snapshot']['description'])
         self.assertIn('updated_at', resp_dict['snapshot'])
 
+        db.volume_destroy(self.ctx, volume.id)
+
+    @ddt.data(False, 'n', 'false', 'No', '0', 'off')
+    def test_snapshot_create_force_failure(self, force_param):
+        volume = utils.create_volume(self.ctx, status='in-use',
+                                     volume_type_id=None)
+        snapshot_name = 'Snapshot Test Name'
+        snapshot_description = 'Snapshot Test Desc'
         snapshot = {
             "volume_id": volume.id,
-            "force": "**&&^^%%$$##@@",
-            "name": "Snapshot Test Name",
-            "description": "Snapshot Test Desc"
+            "force": force_param,
+            "name": snapshot_name,
+            "description": snapshot_description
         }
         body = dict(snapshot=snapshot)
         req = fakes.HTTPRequest.blank('/v2/snapshots')
-        self.assertRaises(exception.InvalidParameterValue,
+        self.assertRaises(exception.InvalidVolume,
                           self.controller.create,
                           req,
-                          body)
+                          body=body)
+
+        db.volume_destroy(self.ctx, volume.id)
+
+    @ddt.data("**&&^^%%$$##@@", '-1', 2, '01', 'falSE', 0, 'trUE', 1,
+              "1         ")
+    def test_snapshot_create_invalid_force_param(self, force_param):
+        volume = utils.create_volume(self.ctx, status='available',
+                                     volume_type_id=None)
+        snapshot_name = 'Snapshot Test Name'
+        snapshot_description = 'Snapshot Test Desc'
+
+        snapshot = {
+            "volume_id": volume.id,
+            "force": force_param,
+            "name": snapshot_name,
+            "description": snapshot_description
+        }
+        body = dict(snapshot=snapshot)
+        req = fakes.HTTPRequest.blank('/v2/snapshots')
+        self.assertRaises(exception.ValidationError,
+                          self.controller.create,
+                          req,
+                          body=body)
 
         db.volume_destroy(self.ctx, volume.id)
 
@@ -160,23 +211,40 @@ class SnapshotApiTest(test.TestCase):
             }
         }
         req = fakes.HTTPRequest.blank('/v2/snapshots')
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.create, req, body)
+        self.assertRaises(exception.ValidationError,
+                          self.controller.create, req, body=body)
+
+    @ddt.data({"snapshot": {"description": "   sample description",
+                            "name": "   test"}},
+              {"snapshot": {"description": "sample description   ",
+                            "name": "test   "}},
+              {"snapshot": {"description": " sample description ",
+                            "name": "  test name  "}})
+    def test_snapshot_create_with_leading_trailing_spaces(self, body):
+        volume = utils.create_volume(self.ctx, volume_type_id=None)
+        body['snapshot']['volume_id'] = volume.id
+        req = fakes.HTTPRequest.blank('/v2/snapshots')
+        resp_dict = self.controller.create(req, body=body)
+
+        self.assertEqual(body['snapshot']['display_name'].strip(),
+                         resp_dict['snapshot']['name'])
+        self.assertEqual(body['snapshot']['description'].strip(),
+                         resp_dict['snapshot']['description'])
+        db.volume_destroy(self.ctx, volume.id)
 
     @mock.patch.object(volume.api.API, "update_snapshot",
-                       side_effect=stubs.stub_snapshot_update)
+                       side_effect=v2_fakes.fake_snapshot_update)
     @mock.patch('cinder.db.snapshot_metadata_get', return_value=dict())
-    @mock.patch('cinder.objects.Volume.get_by_id')
+    @mock.patch('cinder.db.volume_get')
     @mock.patch('cinder.objects.Snapshot.get_by_id')
-    @mock.patch(
-        'cinder.api.openstack.wsgi.Controller.validate_name_and_description')
     def test_snapshot_update(
-            self, mock_validate, snapshot_get_by_id, volume_get_by_id,
+            self, snapshot_get_by_id, volume_get,
             snapshot_metadata_get, update_snapshot):
         snapshot = {
             'id': UUID,
             'volume_id': fake.VOLUME_ID,
             'status': fields.SnapshotStatus.AVAILABLE,
+            'created_at': "2014-01-01 00:00:00",
             'volume_size': 100,
             'display_name': 'Default name',
             'display_description': 'Default description',
@@ -186,21 +254,22 @@ class SnapshotApiTest(test.TestCase):
         snapshot_obj = fake_snapshot.fake_snapshot_obj(ctx, **snapshot)
         fake_volume_obj = fake_volume.fake_volume_obj(ctx)
         snapshot_get_by_id.return_value = snapshot_obj
-        volume_get_by_id.return_value = fake_volume_obj
+        volume_get.return_value = fake_volume_obj
 
         updates = {
             "name": "Updated Test Name",
         }
         body = {"snapshot": updates}
         req = fakes.HTTPRequest.blank('/v2/snapshots/%s' % UUID)
-        res_dict = self.controller.update(req, UUID, body)
+        res_dict = self.controller.update(req, UUID, body=body)
         expected = {
             'snapshot': {
                 'id': UUID,
                 'volume_id': fake.VOLUME_ID,
                 'status': fields.SnapshotStatus.AVAILABLE,
                 'size': 100,
-                'created_at': None,
+                'created_at': datetime.datetime(2014, 1, 1, 0, 0, 0,
+                                                tzinfo=pytz.utc),
                 'updated_at': None,
                 'name': u'Updated Test Name',
                 'description': u'Default description',
@@ -208,33 +277,117 @@ class SnapshotApiTest(test.TestCase):
             }
         }
         self.assertEqual(expected, res_dict)
-        self.assertTrue(mock_validate.called)
         self.assertEqual(2, len(self.notifier.notifications))
 
+    @mock.patch.object(volume.api.API, "update_snapshot",
+                       side_effect=v2_fakes.fake_snapshot_update)
+    @mock.patch('cinder.db.snapshot_metadata_get', return_value=dict())
+    @mock.patch('cinder.db.volume_get')
+    @mock.patch('cinder.objects.Snapshot.get_by_id')
+    def test_snapshot_update_with_null_validate(
+            self, snapshot_get_by_id, volume_get,
+            snapshot_metadata_get, update_snapshot):
+        snapshot = {
+            'id': UUID,
+            'volume_id': fake.VOLUME_ID,
+            'status': fields.SnapshotStatus.AVAILABLE,
+            'created_at': "2014-01-01 00:00:00",
+            'volume_size': 100,
+            'name': 'Default name',
+            'description': 'Default description',
+            'expected_attrs': ['metadata'],
+        }
+        ctx = context.RequestContext(fake.PROJECT_ID, fake.USER_ID, True)
+        snapshot_obj = fake_snapshot.fake_snapshot_obj(ctx, **snapshot)
+        fake_volume_obj = fake_volume.fake_volume_obj(ctx)
+        snapshot_get_by_id.return_value = snapshot_obj
+        volume_get.return_value = fake_volume_obj
+
+        updates = {
+            "name": None,
+            "description": None,
+        }
+        body = {"snapshot": updates}
+        req = fakes.HTTPRequest.blank('/v2/snapshots/%s' % UUID)
+        res_dict = self.controller.update(req, UUID, body=body)
+
+        self.assertEqual(fields.SnapshotStatus.AVAILABLE,
+                         res_dict['snapshot']['status'])
+        self.assertIsNone(res_dict['snapshot']['name'])
+        self.assertIsNone(res_dict['snapshot']['description'])
+
     def test_snapshot_update_missing_body(self):
         body = {}
         req = fakes.HTTPRequest.blank('/v2/snapshots/%s' % UUID)
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.update, req, UUID, body)
+        self.assertRaises(exception.ValidationError,
+                          self.controller.update, req, UUID, body=body)
 
     def test_snapshot_update_invalid_body(self):
         body = {'name': 'missing top level snapshot key'}
         req = fakes.HTTPRequest.blank('/v2/snapshots/%s' % UUID)
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.update, req, UUID, body)
+        self.assertRaises(exception.ValidationError,
+                          self.controller.update, req, UUID, body=body)
 
     def test_snapshot_update_not_found(self):
-        self.stubs.Set(volume.api.API, "get_snapshot", stub_snapshot_get)
+        self.mock_object(volume.api.API, "get_snapshot", fake_snapshot_get)
         updates = {
             "name": "Updated Test Name",
         }
         body = {"snapshot": updates}
         req = fakes.HTTPRequest.blank('/v2/snapshots/not-the-uuid')
         self.assertRaises(exception.SnapshotNotFound, self.controller.update,
-                          req, 'not-the-uuid', body)
+                          req, 'not-the-uuid', body=body)
+
+    @mock.patch.object(volume.api.API, "update_snapshot",
+                       side_effect=v2_fakes.fake_snapshot_update)
+    @mock.patch('cinder.db.snapshot_metadata_get', return_value=dict())
+    @mock.patch('cinder.db.volume_get')
+    @mock.patch('cinder.objects.Snapshot.get_by_id')
+    def test_snapshot_update_with_leading_trailing_spaces(
+            self, snapshot_get_by_id, volume_get,
+            snapshot_metadata_get, update_snapshot):
+        snapshot = {
+            'id': UUID,
+            'volume_id': fake.VOLUME_ID,
+            'status': fields.SnapshotStatus.AVAILABLE,
+            'created_at': "2018-01-14 00:00:00",
+            'volume_size': 100,
+            'display_name': 'Default name',
+            'display_description': 'Default description',
+            'expected_attrs': ['metadata'],
+        }
+        ctx = context.RequestContext(fake.PROJECT_ID, fake.USER_ID, True)
+        snapshot_obj = fake_snapshot.fake_snapshot_obj(ctx, **snapshot)
+        fake_volume_obj = fake_volume.fake_volume_obj(ctx)
+        snapshot_get_by_id.return_value = snapshot_obj
+        volume_get.return_value = fake_volume_obj
+
+        updates = {
+            "name": "     test     ",
+            "description": "     test     "
+        }
+        body = {"snapshot": updates}
+        req = fakes.HTTPRequest.blank('/v2/snapshots/%s' % UUID)
+        res_dict = self.controller.update(req, UUID, body=body)
+        expected = {
+            'snapshot': {
+                'id': UUID,
+                'volume_id': fake.VOLUME_ID,
+                'status': fields.SnapshotStatus.AVAILABLE,
+                'size': 100,
+                'created_at': datetime.datetime(2018, 1, 14, 0, 0, 0,
+                                                tzinfo=pytz.utc),
+                'updated_at': None,
+                'name': u'test',
+                'description': u'test',
+                'metadata': {},
+            }
+        }
+        self.assertEqual(expected, res_dict)
+        self.assertEqual(2, len(self.notifier.notifications))
 
     @mock.patch.object(volume.api.API, "delete_snapshot",
-                       side_effect=stubs.stub_snapshot_update)
+                       side_effect=v2_fakes.fake_snapshot_update)
     @mock.patch('cinder.db.snapshot_metadata_get', return_value=dict())
     @mock.patch('cinder.objects.Volume.get_by_id')
     @mock.patch('cinder.objects.Snapshot.get_by_id')
@@ -258,10 +411,11 @@ class SnapshotApiTest(test.TestCase):
         snapshot_id = UUID
         req = fakes.HTTPRequest.blank('/v2/snapshots/%s' % snapshot_id)
         resp = self.controller.delete(req, snapshot_id)
-        self.assertEqual(202, resp.status_int)
+        self.assertEqual(http_client.ACCEPTED, resp.status_int)
 
     def test_snapshot_delete_invalid_id(self):
-        self.stubs.Set(volume.api.API, "delete_snapshot", stub_snapshot_delete)
+        self.mock_object(volume.api.API, "delete_snapshot",
+                         fake_snapshot_delete)
         snapshot_id = INVALID_UUID
         req = fakes.HTTPRequest.blank('/v2/snapshots/%s' % snapshot_id)
         self.assertRaises(exception.SnapshotNotFound, self.controller.delete,
@@ -333,6 +487,10 @@ class SnapshotApiTest(test.TestCase):
         resp_snapshot = resp_snapshots.pop()
         self.assertEqual(UUID, resp_snapshot['id'])
 
+    @mock.patch.object(db, 'snapshot_get_all_by_project',
+                       v2_fakes.fake_snapshot_get_all_by_project)
+    @mock.patch.object(db, 'snapshot_get_all',
+                       v2_fakes.fake_snapshot_get_all)
     @mock.patch('cinder.db.snapshot_metadata_get', return_value=dict())
     def test_admin_list_snapshots_limited_to_project(self,
                                                      snapshot_metadata_get):
@@ -362,7 +520,6 @@ class SnapshotApiTest(test.TestCase):
             req = fakes.HTTPRequest.blank('/v2/snapshots?limit=1&offset=3')
             self.assertEqual({'snapshots': []}, self.controller.index(req))
 
-        self.stubs.UnsetAll()
         volume, snaps = self._create_db_snapshots(3)
         # admin case
         list_snapshots_with_limit_and_offset(snaps, is_admin=True)
@@ -434,7 +591,7 @@ class SnapshotApiTest(test.TestCase):
             links = res['snapshots_links']
 
             # Must be a list of links, even if we only get 1 back
-            self.assertTrue(list, type(links))
+            self.assertIsInstance(links, list)
             next_link = links[0]
 
             # rel entry must be next
@@ -456,7 +613,7 @@ class SnapshotApiTest(test.TestCase):
             self.assertNotIn('snapshots_links', res)
 
     def _create_db_snapshots(self, num_snaps):
-        volume = utils.create_volume(self.ctx)
+        volume = utils.create_volume(self.ctx, volume_type_id=None)
         snaps = [utils.create_snapshot(self.ctx,
                                        volume.id,
                                        display_name='snap' + str(i))
@@ -471,7 +628,6 @@ class SnapshotApiTest(test.TestCase):
 
     def test_list_snapshots_next_link_default_limit(self):
         """Test that snapshot list pagination is limited by osapi_max_limit."""
-        self.stubs.UnsetAll()
         volume, snaps = self._create_db_snapshots(3)
 
         # NOTE(geguileo): Since cinder.api.common.limited has already been
@@ -502,7 +658,6 @@ class SnapshotApiTest(test.TestCase):
 
     def test_list_snapshots_next_link_with_limit(self):
         """Test snapshot list pagination with specific limit."""
-        self.stubs.UnsetAll()
         volume, snaps = self._create_db_snapshots(2)
 
         # The link from the first page should link to the second
@@ -518,6 +673,10 @@ class SnapshotApiTest(test.TestCase):
         # next links
         self._assert_list_next(limit=1, marker=snaps[1].id)
 
+    @mock.patch.object(db, 'snapshot_get_all_by_project',
+                       v2_fakes.fake_snapshot_get_all_by_project)
+    @mock.patch.object(db, 'snapshot_get_all',
+                       v2_fakes.fake_snapshot_get_all)
     @mock.patch('cinder.db.snapshot_metadata_get', return_value=dict())
     def test_admin_list_snapshots_all_tenants(self, snapshot_metadata_get):
         req = fakes.HTTPRequest.blank('/v2/%s/snapshots?all_tenants=1' %
@@ -534,8 +693,8 @@ class SnapshotApiTest(test.TestCase):
         def get_all(context, filters=None, marker=None, limit=None,
                     sort_keys=None, sort_dirs=None, offset=None):
             if 'project_id' in filters and 'tenant1' in filters['project_id']:
-                return [stubs.stub_snapshot(fake.VOLUME_ID,
-                                            tenant_id='tenant1')]
+                return [v2_fakes.fake_snapshot(fake.VOLUME_ID,
+                                               tenant_id='tenant1')]
             else:
                 return []
 
@@ -548,6 +707,8 @@ class SnapshotApiTest(test.TestCase):
         self.assertIn('snapshots', res)
         self.assertEqual(1, len(res['snapshots']))
 
+    @mock.patch.object(db, 'snapshot_get_all_by_project',
+                       v2_fakes.fake_snapshot_get_all_by_project)
     @mock.patch('cinder.db.snapshot_metadata_get', return_value=dict())
     def test_all_tenants_non_admin_gets_all_tenants(self,
                                                     snapshot_metadata_get):
@@ -557,6 +718,10 @@ class SnapshotApiTest(test.TestCase):
         self.assertIn('snapshots', res)
         self.assertEqual(1, len(res['snapshots']))
 
+    @mock.patch.object(db, 'snapshot_get_all_by_project',
+                       v2_fakes.fake_snapshot_get_all_by_project)
+    @mock.patch.object(db, 'snapshot_get_all',
+                       v2_fakes.fake_snapshot_get_all)
     @mock.patch('cinder.db.snapshot_metadata_get', return_value=dict())
     def test_non_admin_get_by_project(self, snapshot_metadata_get):
         req = fakes.HTTPRequest.blank('/v2/%s/snapshots' % fake.PROJECT_ID)
@@ -568,8 +733,8 @@ class SnapshotApiTest(test.TestCase):
         req = fakes.HTTPRequest.blank('/v2/%s/snapshots' % fake.PROJECT_ID)
         req.method = 'POST'
 
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.create, req, body)
+        self.assertRaises(exception.ValidationError,
+                          self.controller.create, req, body=body)
 
     def test_create_no_body(self):
         self._create_snapshot_bad_body(body=None)
