@@ -13,8 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+
 import ddt
-import mock
 from oslo_utils import strutils
 
 from cinder.api import microversions as mv
@@ -24,12 +25,12 @@ from cinder import db
 from cinder import exception
 from cinder.objects import fields
 from cinder.scheduler import rpcapi as scheduler_rpcapi
-from cinder import test
 from cinder.tests.unit.api import fakes
 from cinder.tests.unit.api.v2 import fakes as v2_fakes
 from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import fake_snapshot
 from cinder.tests.unit import fake_volume
+from cinder.tests.unit import test
 from cinder.tests.unit import utils as test_utils
 from cinder import volume
 
@@ -44,7 +45,6 @@ def fake_get(self, context, *args, **kwargs):
            'host': 'fake-host',
            'status': 'available',
            'encryption_key_id': None,
-           'volume_type_id': None,
            'migration_status': None,
            'availability_zone': 'fake-zone',
            'attach_status': 'detached',
