@@ -16,14 +16,15 @@
 #    under the License.
 
 import datetime
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 from oslo_config import fixture as config_fixture
 from oslo_utils import timeutils
 import six
 
 from cinder import backup
+from cinder.backup import api as backup_api
 from cinder import context
 from cinder import db
 from cinder.db.sqlalchemy import api as sqa_api
@@ -33,9 +34,11 @@ from cinder import objects
 from cinder.objects import fields
 from cinder import quota
 from cinder import quota_utils
-from cinder import test
+from cinder.scheduler import rpcapi as scheduler_rpcapi
 from cinder.tests.unit import fake_constants as fake
 import cinder.tests.unit.image.fake
+from cinder.tests.unit import test
+from cinder.tests.unit import utils as tests_utils
 from cinder import volume
 
 
@@ -48,11 +51,9 @@ class QuotaIntegrationTestCase(test.TestCase):
         objects.register_all()
         super(QuotaIntegrationTestCase, self).setUp()
         self.volume_type_name = CONF.default_volume_type
-        self.volume_type = db.volume_type_create(
+        self.volume_type = objects.VolumeType.get_by_name_or_id(
             context.get_admin_context(),
-            dict(name=self.volume_type_name))
-        self.addCleanup(db.volume_type_destroy, context.get_admin_context(),
-                        self.volume_type['id'])
+            identity=self.volume_type_name)
 
         self.flags(quota_volumes=2,
                    quota_snapshots=2,
@@ -82,7 +83,7 @@ class QuotaIntegrationTestCase(test.TestCase):
         vol['volume_type_id'] = self.volume_type['id']
         vol['host'] = 'fake_host'
         vol['availability_zone'] = 'fake_zone'
-        vol['attach_status'] = 'detached'
+        vol['attach_status'] = fields.VolumeAttachStatus.DETACHED
         volume = objects.Volume(context=self.context, **vol)
         volume.create()
         return volume
@@ -152,6 +153,27 @@ class QuotaIntegrationTestCase(test.TestCase):
         self.assertEqual(msg, six.text_type(ex))
         vol_ref.destroy()
 
+    def test__snapshots_quota_value(self):
+        test_volume1 = tests_utils.create_volume(
+            self.context,
+            status='available',
+            host=CONF.host,
+            volume_type_id=self.vt['id'])
+        test_volume2 = tests_utils.create_volume(
+            self.context,
+            status='available',
+            host=CONF.host,
+            volume_type_id=self.vt['id'])
+        volume_api = volume.api.API()
+        volume_api.create_snapshots_in_db(self.context,
+                                          [test_volume1, test_volume2],
+                                          'fake_name',
+                                          'fake_description',
+                                          fake.CONSISTENCY_GROUP_ID)
+        usages = db.quota_usage_get_all_by_project(self.context,
+                                                   self.project_id)
+        self.assertEqual(2, usages['snapshots']['in_use'])
+
     def test_too_many_snapshots_of_type(self):
         resource = 'snapshots_%s' % self.volume_type_name
         db.quota_class_create(self.context, 'default', resource, 1)
@@ -178,7 +200,7 @@ class QuotaIntegrationTestCase(test.TestCase):
         self.flags(**flag_args)
         vol_ref = self._create_volume()
         backup_ref = self._create_backup(vol_ref)
-        with mock.patch.object(backup.API,
+        with mock.patch.object(backup_api.API,
                                '_get_available_backup_service_host') as \
                 mock__get_available_backup_service:
             mock__get_available_backup_service.return_value = 'host'
@@ -222,7 +244,7 @@ class QuotaIntegrationTestCase(test.TestCase):
     def test_too_many_combined_backup_gigabytes(self):
         vol_ref = self._create_volume(size=10000)
         backup_ref = self._create_backup(vol_ref)
-        with mock.patch.object(backup.API,
+        with mock.patch.object(backup_api.API,
                                '_get_available_backup_service_host') as \
                 mock__get_available_backup_service:
             mock__get_available_backup_service.return_value = 'host'
@@ -239,6 +261,7 @@ class QuotaIntegrationTestCase(test.TestCase):
             vol_ref.destroy()
 
     def test_no_snapshot_gb_quota_flag(self):
+        self.mock_object(scheduler_rpcapi.SchedulerAPI, 'create_snapshot')
         self.flags(quota_volumes=2,
                    quota_snapshots=2,
                    quota_gigabytes=20,
@@ -268,7 +291,7 @@ class QuotaIntegrationTestCase(test.TestCase):
                    )
         vol_ref = self._create_volume(size=10)
         backup_ref = self._create_backup(vol_ref)
-        with mock.patch.object(backup.API,
+        with mock.patch.object(backup_api.API,
                                '_get_available_backup_service_host') as \
                 mock_mock__get_available_backup_service:
             mock_mock__get_available_backup_service.return_value = 'host'
@@ -838,7 +861,7 @@ class VolumeTypeQuotaEngineTestCase(test.TestCase):
     def test_default_resources(self):
         def fake_vtga(context, inactive=False, filters=None):
             return {}
-        self.stubs.Set(db, 'volume_type_get_all', fake_vtga)
+        self.mock_object(db, 'volume_type_get_all', fake_vtga)
 
         engine = quota.VolumeTypeQuotaEngine()
         self.assertEqual(['backup_gigabytes', 'backups',
@@ -864,7 +887,7 @@ class VolumeTypeQuotaEngineTestCase(test.TestCase):
                     'extra_specs': {},
                 },
             }
-        self.stubs.Set(db, 'volume_type_get_all', fake_vtga)
+        self.mock_object(db, 'volume_type_get_all', fake_vtga)
 
         engine = quota.VolumeTypeQuotaEngine()
         self.assertEqual(['backup_gigabytes', 'backups',
@@ -920,40 +943,40 @@ class DbQuotaDriverBaseTestCase(test.TestCase):
         self.mock_utcnow = patcher.start()
         self.mock_utcnow.return_value = datetime.datetime.utcnow()
 
-    def _stub_quota_class_get_default(self):
-        # Stub out quota_class_get_default
+    def _mock_quota_class_get_default(self):
+        # Mock quota_class_get_default
         def fake_qcgd(context):
-            self.calls.append('quota_class_get_default')
+            self.calls.append('quota_class_get_defaults')
             return dict(volumes=10,
                         snapshots=10,
                         gigabytes=1000,
                         backups=10,
                         backup_gigabytes=1000
                         )
-        self.stubs.Set(db, 'quota_class_get_default', fake_qcgd)
+        self.mock_object(db, 'quota_class_get_defaults', fake_qcgd)
 
-    def _stub_volume_type_get_all(self):
+    def _mock_volume_type_get_all(self):
         def fake_vtga(context, inactive=False, filters=None):
             return {}
-        self.stubs.Set(db, 'volume_type_get_all', fake_vtga)
+        self.mock_object(db, 'volume_type_get_all', fake_vtga)
 
-    def _stub_quota_class_get_all_by_name(self):
-        # Stub out quota_class_get_all_by_name
+    def _mock_quota_class_get_all_by_name(self):
+        # Mock quota_class_get_all_by_name
         def fake_qcgabn(context, quota_class):
             self.calls.append('quota_class_get_all_by_name')
             self.assertEqual('test_class', quota_class)
             return dict(gigabytes=500, volumes=10, snapshots=10, backups=10,
                         backup_gigabytes=500)
-        self.stubs.Set(db, 'quota_class_get_all_by_name', fake_qcgabn)
+        self.mock_object(db, 'quota_class_get_all_by_name', fake_qcgabn)
 
-    def _stub_allocated_get_all_by_project(self, allocated_quota=False):
-        def fake_qagabp(context, project_id):
+    def _mock_allocated_get_all_by_project(self, allocated_quota=False):
+        def fake_qagabp(context, project_id, session=None):
             self.calls.append('quota_allocated_get_all_by_project')
             if allocated_quota:
                 return dict(project_id=project_id, volumes=3)
             return dict(project_id=project_id)
 
-        self.stubs.Set(db, 'quota_allocated_get_all_by_project', fake_qagabp)
+        self.mock_object(db, 'quota_allocated_get_all_by_project', fake_qagabp)
 
 
 class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
@@ -964,8 +987,8 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
 
     def test_get_defaults(self):
         # Use our pre-defined resources
-        self._stub_quota_class_get_default()
-        self._stub_volume_type_get_all()
+        self._mock_quota_class_get_default()
+        self._mock_volume_type_get_all()
         result = self.driver.get_defaults(None, quota.QUOTAS.resources)
 
         self.assertEqual(
@@ -978,8 +1001,8 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
                 per_volume_gigabytes=-1), result)
 
     def test_get_class_quotas(self):
-        self._stub_quota_class_get_all_by_name()
-        self._stub_volume_type_get_all()
+        self._mock_quota_class_get_all_by_name()
+        self._mock_volume_type_get_all()
         result = self.driver.get_class_quotas(None, quota.QUOTAS.resources,
                                               'test_class')
 
@@ -992,7 +1015,7 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
                          per_volume_gigabytes=-1), result)
 
     def test_get_class_quotas_no_defaults(self):
-        self._stub_quota_class_get_all_by_name()
+        self._mock_quota_class_get_all_by_name()
         result = self.driver.get_class_quotas(None, quota.QUOTAS.resources,
                                               'test_class', False)
 
@@ -1003,7 +1026,7 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
                               backups=10,
                               backup_gigabytes=500), result)
 
-    def _stub_get_by_project(self):
+    def _mock_get_by_project(self):
         def fake_qgabp(context, project_id):
             self.calls.append('quota_get_all_by_project')
             self.assertEqual('test_project', project_id)
@@ -1021,16 +1044,16 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
                         backup_gigabytes=dict(in_use=10, reserved=0)
                         )
 
-        self.stubs.Set(db, 'quota_get_all_by_project', fake_qgabp)
-        self.stubs.Set(db, 'quota_usage_get_all_by_project', fake_qugabp)
+        self.mock_object(db, 'quota_get_all_by_project', fake_qgabp)
+        self.mock_object(db, 'quota_usage_get_all_by_project', fake_qugabp)
 
-        self._stub_quota_class_get_all_by_name()
-        self._stub_quota_class_get_default()
+        self._mock_quota_class_get_all_by_name()
+        self._mock_quota_class_get_default()
 
     def test_get_project_quotas(self):
-        self._stub_get_by_project()
-        self._stub_volume_type_get_all()
-        self._stub_allocated_get_all_by_project()
+        self._mock_get_by_project()
+        self._mock_volume_type_get_all()
+        self._mock_allocated_get_all_by_project()
         result = self.driver.get_project_quotas(
             FakeContext('test_project', 'test_class'),
             quota.QUOTAS.resources, 'test_project')
@@ -1039,7 +1062,7 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
                           'quota_usage_get_all_by_project',
                           'quota_allocated_get_all_by_project',
                           'quota_class_get_all_by_name',
-                          'quota_class_get_default', ], self.calls)
+                          'quota_class_get_defaults', ], self.calls)
         self.assertEqual(dict(volumes=dict(limit=10,
                                            in_use=2,
                                            reserved=0, ),
@@ -1061,10 +1084,17 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
                               ), result)
 
     @mock.patch('cinder.quota.db.quota_get_all_by_project')
-    @mock.patch('cinder.quota.db.quota_class_get_default')
+    @mock.patch('cinder.quota.db.quota_class_get_defaults')
     def test_get_project_quotas_lazy_load_defaults(
             self, mock_defaults, mock_quotas):
-        mock_quotas.return_value = self._default_quotas_non_child
+        defaults = self._default_quotas_non_child
+        volume_types = volume.volume_types.get_all_types(
+            context.get_admin_context())
+        for vol_type in volume_types:
+            defaults['volumes_' + vol_type] = -1
+            defaults['snapshots_' + vol_type] = -1
+            defaults['gigabytes_' + vol_type] = -1
+        mock_quotas.return_value = defaults
         self.driver.get_project_quotas(
             FakeContext('test_project', None),
             quota.QUOTAS.resources, 'test_project', usages=False)
@@ -1079,9 +1109,9 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
         self.assertTrue(mock_defaults.called)
 
     def test_get_root_project_with_subprojects_quotas(self):
-        self._stub_get_by_project()
-        self._stub_volume_type_get_all()
-        self._stub_allocated_get_all_by_project(allocated_quota=True)
+        self._mock_get_by_project()
+        self._mock_volume_type_get_all()
+        self._mock_allocated_get_all_by_project(allocated_quota=True)
         result = self.driver.get_project_quotas(
             FakeContext('test_project', None),
             quota.QUOTAS.resources, 'test_project')
@@ -1089,7 +1119,7 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
         self.assertEqual(['quota_get_all_by_project',
                           'quota_usage_get_all_by_project',
                           'quota_allocated_get_all_by_project',
-                          'quota_class_get_default', ], self.calls)
+                          'quota_class_get_defaults', ], self.calls)
         self.assertEqual(dict(volumes=dict(limit=10,
                                            in_use=2,
                                            reserved=0,
@@ -1117,15 +1147,15 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
                               ), result)
 
     def test_get_project_quotas_alt_context_no_class(self):
-        self._stub_get_by_project()
-        self._stub_volume_type_get_all()
+        self._mock_get_by_project()
+        self._mock_volume_type_get_all()
         result = self.driver.get_project_quotas(
             FakeContext('other_project', 'other_class'),
             quota.QUOTAS.resources, 'test_project')
 
         self.assertEqual(['quota_get_all_by_project',
                           'quota_usage_get_all_by_project',
-                          'quota_class_get_default', ], self.calls)
+                          'quota_class_get_defaults', ], self.calls)
         self.assertEqual(dict(volumes=dict(limit=10,
                                            in_use=2,
                                            reserved=0, ),
@@ -1147,8 +1177,8 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
                               ), result)
 
     def test_get_project_quotas_alt_context_with_class(self):
-        self._stub_get_by_project()
-        self._stub_volume_type_get_all()
+        self._mock_get_by_project()
+        self._mock_volume_type_get_all()
         result = self.driver.get_project_quotas(
             FakeContext('other_project', 'other_class'),
             quota.QUOTAS.resources, 'test_project', quota_class='test_class')
@@ -1156,7 +1186,7 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
         self.assertEqual(['quota_get_all_by_project',
                           'quota_usage_get_all_by_project',
                           'quota_class_get_all_by_name',
-                          'quota_class_get_default', ], self.calls)
+                          'quota_class_get_defaults', ], self.calls)
         self.assertEqual(dict(volumes=dict(limit=10,
                                            in_use=2,
                                            reserved=0, ),
@@ -1178,8 +1208,8 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
                          result)
 
     def test_get_project_quotas_no_defaults(self):
-        self._stub_get_by_project()
-        self._stub_volume_type_get_all()
+        self._mock_get_by_project()
+        self._mock_volume_type_get_all()
         result = self.driver.get_project_quotas(
             FakeContext('test_project', 'test_class'),
             quota.QUOTAS.resources, 'test_project', defaults=False)
@@ -1206,15 +1236,15 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
                               ), result)
 
     def test_get_project_quotas_no_usages(self):
-        self._stub_get_by_project()
-        self._stub_volume_type_get_all()
+        self._mock_get_by_project()
+        self._mock_volume_type_get_all()
         result = self.driver.get_project_quotas(
             FakeContext('test_project', 'test_class'),
             quota.QUOTAS.resources, 'test_project', usages=False)
 
         self.assertEqual(['quota_get_all_by_project',
                           'quota_class_get_all_by_name',
-                          'quota_class_get_default', ], self.calls)
+                          'quota_class_get_defaults', ], self.calls)
         self.assertEqual(dict(volumes=dict(limit=10, ),
                               snapshots=dict(limit=10, ),
                               backups=dict(limit=10, ),
@@ -1222,18 +1252,18 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
                               backup_gigabytes=dict(limit=50, ),
                               per_volume_gigabytes=dict(limit=-1, )), result)
 
-    def _stub_get_project_quotas(self):
+    def _mock_get_project_quotas(self):
         def fake_get_project_quotas(context, resources, project_id,
                                     quota_class=None, defaults=True,
                                     usages=True, parent_project_id=None):
             self.calls.append('get_project_quotas')
             return {k: dict(limit=v.default) for k, v in resources.items()}
 
-        self.stubs.Set(self.driver, 'get_project_quotas',
-                       fake_get_project_quotas)
+        self.mock_object(self.driver, 'get_project_quotas',
+                         fake_get_project_quotas)
 
     def test_get_quotas_has_sync_unknown(self):
-        self._stub_get_project_quotas()
+        self._mock_get_project_quotas()
         self.assertRaises(exception.QuotaResourceUnknown,
                           self.driver._get_quotas,
                           None, quota.QUOTAS.resources,
@@ -1241,7 +1271,7 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
         self.assertEqual([], self.calls)
 
     def test_get_quotas_no_sync_unknown(self):
-        self._stub_get_project_quotas()
+        self._mock_get_project_quotas()
         self.assertRaises(exception.QuotaResourceUnknown,
                           self.driver._get_quotas,
                           None, quota.QUOTAS.resources,
@@ -1249,7 +1279,7 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
         self.assertEqual([], self.calls)
 
     def test_get_quotas_has_sync_no_sync_resource(self):
-        self._stub_get_project_quotas()
+        self._mock_get_project_quotas()
         self.assertRaises(exception.QuotaResourceUnknown,
                           self.driver._get_quotas,
                           None, quota.QUOTAS.resources,
@@ -1257,7 +1287,7 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
         self.assertEqual([], self.calls)
 
     def test_get_quotas_no_sync_has_sync_resource(self):
-        self._stub_get_project_quotas()
+        self._mock_get_project_quotas()
         self.assertRaises(exception.QuotaResourceUnknown,
                           self.driver._get_quotas,
                           None, quota.QUOTAS.resources,
@@ -1265,7 +1295,7 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
         self.assertEqual([], self.calls)
 
     def test_get_quotas_has_sync(self):
-        self._stub_get_project_quotas()
+        self._mock_get_project_quotas()
         result = self.driver._get_quotas(FakeContext('test_project',
                                                      'test_class'),
                                          quota.QUOTAS.resources,
@@ -1275,17 +1305,17 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
         self.assertEqual(['get_project_quotas'], self.calls)
         self.assertEqual(dict(volumes=10, gigabytes=1000, ), result)
 
-    def _stub_quota_reserve(self):
+    def _mock_quota_reserve(self):
         def fake_quota_reserve(context, resources, quotas, deltas, expire,
                                until_refresh, max_age, project_id=None):
             self.calls.append(('quota_reserve', expire, until_refresh,
                                max_age))
             return ['resv-1', 'resv-2', 'resv-3']
-        self.stubs.Set(db, 'quota_reserve', fake_quota_reserve)
+        self.mock_object(db, 'quota_reserve', fake_quota_reserve)
 
     def test_reserve_bad_expire(self):
-        self._stub_get_project_quotas()
-        self._stub_quota_reserve()
+        self._mock_get_project_quotas()
+        self._mock_quota_reserve()
         self.assertRaises(exception.InvalidReservationExpiration,
                           self.driver.reserve,
                           FakeContext('test_project', 'test_class'),
@@ -1294,8 +1324,8 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
         self.assertEqual([], self.calls)
 
     def test_reserve_default_expire(self):
-        self._stub_get_project_quotas()
-        self._stub_quota_reserve()
+        self._mock_get_project_quotas()
+        self._mock_quota_reserve()
         result = self.driver.reserve(FakeContext('test_project', 'test_class'),
                                      quota.QUOTAS.resources,
                                      dict(volumes=2))
@@ -1306,8 +1336,8 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
         self.assertEqual(['resv-1', 'resv-2', 'resv-3'], result)
 
     def test_reserve_int_expire(self):
-        self._stub_get_project_quotas()
-        self._stub_quota_reserve()
+        self._mock_get_project_quotas()
+        self._mock_quota_reserve()
         result = self.driver.reserve(FakeContext('test_project', 'test_class'),
                                      quota.QUOTAS.resources,
                                      dict(volumes=2), expire=3600)
@@ -1318,8 +1348,8 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
         self.assertEqual(['resv-1', 'resv-2', 'resv-3'], result)
 
     def test_reserve_timedelta_expire(self):
-        self._stub_get_project_quotas()
-        self._stub_quota_reserve()
+        self._mock_get_project_quotas()
+        self._mock_quota_reserve()
         expire_delta = datetime.timedelta(seconds=60)
         result = self.driver.reserve(FakeContext('test_project', 'test_class'),
                                      quota.QUOTAS.resources,
@@ -1331,8 +1361,8 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
         self.assertEqual(['resv-1', 'resv-2', 'resv-3'], result)
 
     def test_reserve_datetime_expire(self):
-        self._stub_get_project_quotas()
-        self._stub_quota_reserve()
+        self._mock_get_project_quotas()
+        self._mock_quota_reserve()
         expire = timeutils.utcnow() + datetime.timedelta(seconds=120)
         result = self.driver.reserve(FakeContext('test_project', 'test_class'),
                                      quota.QUOTAS.resources,
@@ -1343,8 +1373,8 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
         self.assertEqual(['resv-1', 'resv-2', 'resv-3'], result)
 
     def test_reserve_until_refresh(self):
-        self._stub_get_project_quotas()
-        self._stub_quota_reserve()
+        self._mock_get_project_quotas()
+        self._mock_quota_reserve()
         self.flags(until_refresh=500)
         expire = timeutils.utcnow() + datetime.timedelta(seconds=120)
         result = self.driver.reserve(FakeContext('test_project', 'test_class'),
@@ -1356,8 +1386,8 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
         self.assertEqual(['resv-1', 'resv-2', 'resv-3'], result)
 
     def test_reserve_max_age(self):
-        self._stub_get_project_quotas()
-        self._stub_quota_reserve()
+        self._mock_get_project_quotas()
+        self._mock_quota_reserve()
         self.flags(max_age=86400)
         expire = timeutils.utcnow() + datetime.timedelta(seconds=120)
         result = self.driver.reserve(FakeContext('test_project', 'test_class'),
@@ -1368,15 +1398,15 @@ class DbQuotaDriverTestCase(DbQuotaDriverBaseTestCase):
                           ('quota_reserve', expire, 0, 86400), ], self.calls)
         self.assertEqual(['resv-1', 'resv-2', 'resv-3'], result)
 
-    def _stub_quota_destroy_by_project(self):
+    def _mock_quota_destroy_by_project(self):
         def fake_quota_destroy_by_project(context, project_id):
             self.calls.append(('quota_destroy_by_project', project_id))
             return None
-        self.stubs.Set(sqa_api, 'quota_destroy_by_project',
-                       fake_quota_destroy_by_project)
+        self.mock_object(sqa_api, 'quota_destroy_by_project',
+                         fake_quota_destroy_by_project)
 
     def test_destroy_quota_by_project(self):
-        self._stub_quota_destroy_by_project()
+        self._mock_quota_destroy_by_project()
         self.driver.destroy_by_project(FakeContext('test_project',
                                                    'test_class'),
                                        'test_project')
@@ -1423,10 +1453,10 @@ class NestedDbQuotaDriverBaseTestCase(DbQuotaDriverBaseTestCase):
         self.addCleanup(keystone_patcher.stop)
 
         self.fixture = self.useFixture(config_fixture.Config(CONF))
-        self.fixture.config(auth_uri=self.auth_url, group='keystone_authtoken')
+        self.fixture.config(auth_url=self.auth_url, group='keystone_authtoken')
         self.driver = quota.NestedDbQuotaDriver()
 
-    def _stub_get_by_subproject(self):
+    def _mock_get_by_subproject(self):
         def fake_qgabp(context, project_id):
             self.calls.append('quota_get_all_by_project')
             return dict(volumes=10, gigabytes=50, reserved=0)
@@ -1436,15 +1466,15 @@ class NestedDbQuotaDriverBaseTestCase(DbQuotaDriverBaseTestCase):
             return dict(volumes=dict(in_use=2, reserved=0),
                         gigabytes=dict(in_use=10, reserved=0))
 
-        self.stubs.Set(db, 'quota_get_all_by_project', fake_qgabp)
-        self.stubs.Set(db, 'quota_usage_get_all_by_project', fake_qugabp)
+        self.mock_object(db, 'quota_get_all_by_project', fake_qgabp)
+        self.mock_object(db, 'quota_usage_get_all_by_project', fake_qugabp)
 
-        self._stub_quota_class_get_all_by_name()
+        self._mock_quota_class_get_all_by_name()
 
 
 class NestedDbQuotaDriverTestCase(NestedDbQuotaDriverBaseTestCase):
     def test_get_defaults(self):
-        self._stub_volume_type_get_all()
+        self._mock_volume_type_get_all()
 
         # Test for child project defaults
         result = self.driver.get_defaults(self.context,
@@ -1473,9 +1503,9 @@ class NestedDbQuotaDriverTestCase(NestedDbQuotaDriverBaseTestCase):
                           project_id=self._child_proj_id)
 
     def test_get_subproject_quotas(self):
-        self._stub_get_by_subproject()
-        self._stub_volume_type_get_all()
-        self._stub_allocated_get_all_by_project(allocated_quota=True)
+        self._mock_get_by_subproject()
+        self._mock_volume_type_get_all()
+        self._mock_allocated_get_all_by_project(allocated_quota=True)
         result = self.driver.get_project_quotas(
             self.context,
             quota.QUOTAS.resources, self._child_proj_id)
@@ -1513,7 +1543,7 @@ class NestedDbQuotaDriverTestCase(NestedDbQuotaDriverBaseTestCase):
 class NestedQuotaValidation(NestedDbQuotaDriverBaseTestCase):
     def setUp(self):
         super(NestedQuotaValidation, self).setUp()
-        """
+        r"""
         Quota hierarchy setup like so
         +-----------+
         |           |
@@ -1554,7 +1584,8 @@ class NestedQuotaValidation(NestedDbQuotaDriverBaseTestCase):
     def _fake_quota_usage_get_all_by_project(self, context, project_id):
         return {'volumes': self.proj_vals[project_id]}
 
-    def _fake_quota_allocated_get_all_by_project(self, context, project_id):
+    def _fake_quota_allocated_get_all_by_project(self, context, project_id,
+                                                 session=None):
         ret = {'project_id': project_id}
         proj_val = self.proj_vals[project_id]
         if 'alloc' in proj_val:
@@ -1695,7 +1726,7 @@ class QuotaReserveSqlAlchemyTestCase(test.TestCase):
             QUOTA_SYNC_FUNCTIONS['_sync_%s' % res_name] = make_sync(res_name)
             self.resources[res_name] = res
 
-        self.stubs.Set(sqa_api, 'QUOTA_SYNC_FUNCTIONS', QUOTA_SYNC_FUNCTIONS)
+        self.mock_object(sqa_api, 'QUOTA_SYNC_FUNCTIONS', QUOTA_SYNC_FUNCTIONS)
         self.expire = timeutils.utcnow() + datetime.timedelta(seconds=3600)
 
         self.usages = {}
@@ -1705,7 +1736,8 @@ class QuotaReserveSqlAlchemyTestCase(test.TestCase):
         def fake_get_session():
             return FakeSession()
 
-        def fake_get_quota_usages(context, session, project_id):
+        def fake_get_quota_usages(context, session, project_id,
+                                  resources=None):
             return self.usages.copy()
 
         def fake_quota_usage_create(context, project_id, resource, in_use,
@@ -1730,16 +1762,14 @@ class QuotaReserveSqlAlchemyTestCase(test.TestCase):
 
             return reservation_ref
 
-        def fake_qagabp(context, project_id):
-            self.assertEqual('test_project', project_id)
-            return {'project_id': project_id}
-
-        self.stubs.Set(sqa_api, 'quota_allocated_get_all_by_project',
-                       fake_qagabp)
-        self.stubs.Set(sqa_api, 'get_session', fake_get_session)
-        self.stubs.Set(sqa_api, '_get_quota_usages', fake_get_quota_usages)
-        self.stubs.Set(sqa_api, '_quota_usage_create', fake_quota_usage_create)
-        self.stubs.Set(sqa_api, '_reservation_create', fake_reservation_create)
+        self.mock_object(sqa_api, 'get_session',
+                         fake_get_session)
+        self.mock_object(sqa_api, '_get_quota_usages',
+                         fake_get_quota_usages)
+        self.mock_object(sqa_api, '_quota_usage_create',
+                         fake_quota_usage_create)
+        self.mock_object(sqa_api, '_reservation_create',
+                         fake_reservation_create)
 
         patcher = mock.patch.object(timeutils, 'utcnow')
         self.addCleanup(patcher.stop)
@@ -1819,22 +1849,22 @@ class QuotaReserveSqlAlchemyTestCase(test.TestCase):
 
         self.assertEqual(0, len(reservations))
 
-    def _stub_allocated_get_all_by_project(self, allocated_quota=False):
-        def fake_qagabp(context, project_id):
+    def _mock_allocated_get_all_by_project(self, allocated_quota=False):
+        def fake_qagabp(context, project_id, session=None):
             self.assertEqual('test_project', project_id)
+            self.assertIsNotNone(session)
             if allocated_quota:
                 return dict(project_id=project_id, volumes=3,
                             gigabytes = 2 * 1024)
             return dict(project_id=project_id)
 
-        self.stubs.Set(sqa_api,
-                       'quota_allocated_get_all_by_project',
-                       fake_qagabp)
+        self.mock_object(sqa_api, 'quota_allocated_get_all_by_project',
+                         fake_qagabp)
 
     def test_quota_reserve_with_allocated(self):
         context = FakeContext('test_project', 'test_class')
         # Allocated quota for volume will be updated for 3
-        self._stub_allocated_get_all_by_project(allocated_quota=True)
+        self._mock_allocated_get_all_by_project(allocated_quota=True)
         # Quota limited for volume updated for 10
         quotas = dict(volumes=10,
                       gigabytes=10 * 1024, )
@@ -1853,6 +1883,7 @@ class QuotaReserveSqlAlchemyTestCase(test.TestCase):
              dict(resource='gigabytes',
                   usage_id=self.usages_created['gigabytes'],
                   delta=2 * 1024), ])
+
         # But if we try reserve 8 volumes(more free quota that we have)
         deltas = dict(volumes=8,
                       gigabytes=2 * 1024, )
@@ -1868,7 +1899,7 @@ class QuotaReserveSqlAlchemyTestCase(test.TestCase):
                       gigabytes=10 * 1024, )
         deltas = dict(volumes=2,
                       gigabytes=2 * 1024, )
-        self._stub_allocated_get_all_by_project()
+        self._mock_allocated_get_all_by_project()
         result = sqa_api.quota_reserve(context, self.resources, quotas,
                                        deltas, self.expire, 0, 0)
 
@@ -1902,7 +1933,7 @@ class QuotaReserveSqlAlchemyTestCase(test.TestCase):
                       gigabytes=10 * 1024, )
         deltas = dict(volumes=2,
                       gigabytes=2 * 1024, )
-        self._stub_allocated_get_all_by_project()
+        self._mock_allocated_get_all_by_project()
         result = sqa_api.quota_reserve(context, self.resources, quotas,
                                        deltas, self.expire, 5, 0)
 
@@ -1933,7 +1964,7 @@ class QuotaReserveSqlAlchemyTestCase(test.TestCase):
         context = FakeContext('test_project', 'test_class')
         quotas = dict(volumes=5, gigabytes=10 * 1024, )
         deltas = dict(volumes=2, gigabytes=2 * 1024, )
-        self._stub_allocated_get_all_by_project()
+        self._mock_allocated_get_all_by_project()
         result = sqa_api.quota_reserve(context, self.resources, quotas,
                                        deltas, self.expire, 5, 0)
 
@@ -1969,7 +2000,7 @@ class QuotaReserveSqlAlchemyTestCase(test.TestCase):
         context = FakeContext('test_project', 'test_class')
         quotas = dict(volumes=5, gigabytes=10 * 1024, )
         deltas = dict(volumes=2, gigabytes=2 * 1024, )
-        self._stub_allocated_get_all_by_project()
+        self._mock_allocated_get_all_by_project()
         result = sqa_api.quota_reserve(context, self.resources, quotas,
                                        deltas, self.expire, 0, max_age)
 
@@ -1994,13 +2025,49 @@ class QuotaReserveSqlAlchemyTestCase(test.TestCase):
                                        usage_id=self.usages['gigabytes'],
                                        delta=2 * 1024), ])
 
+    def test_quota_reserve_max_age_negative(self):
+        max_age = 3600
+        record_created = (timeutils.utcnow() +
+                          datetime.timedelta(seconds=max_age))
+        self.init_usage('test_project', 'volumes', 3, 0,
+                        created_at=record_created, updated_at=record_created)
+        self.init_usage('test_project', 'gigabytes', 3, 0,
+                        created_at=record_created, updated_at=record_created)
+        context = FakeContext('test_project', 'test_class')
+        quotas = dict(volumes=5, gigabytes=10 * 1024, )
+        deltas = dict(volumes=2, gigabytes=2 * 1024, )
+        self._mock_allocated_get_all_by_project()
+        result = sqa_api.quota_reserve(context, self.resources, quotas,
+                                       deltas, self.expire, 0, max_age)
+
+        self.assertEqual(set(), self.sync_called)
+        self.compare_usage(self.usages, [dict(resource='volumes',
+                                              project_id='test_project',
+                                              in_use=3,
+                                              reserved=2,
+                                              until_refresh=None),
+                                         dict(resource='gigabytes',
+                                              project_id='test_project',
+                                              in_use=3,
+                                              reserved=2 * 1024,
+                                              until_refresh=None), ])
+        self.assertEqual({}, self.usages_created)
+        self.compare_reservation(result,
+                                 [dict(resource='volumes',
+                                       usage_id=self.usages['volumes'],
+                                       project_id='test_project',
+                                       delta=2),
+                                  dict(resource='gigabytes',
+                                       usage_id=self.usages['gigabytes'],
+                                       delta=2 * 1024), ])
+
     def test_quota_reserve_no_refresh(self):
         self.init_usage('test_project', 'volumes', 3, 0)
         self.init_usage('test_project', 'gigabytes', 3, 0)
         context = FakeContext('test_project', 'test_class')
         quotas = dict(volumes=5, gigabytes=10 * 1024, )
         deltas = dict(volumes=2, gigabytes=2 * 1024, )
-        self._stub_allocated_get_all_by_project()
+        self._mock_allocated_get_all_by_project()
         result = sqa_api.quota_reserve(context, self.resources, quotas,
                                        deltas, self.expire, 0, 0)
 
@@ -2031,7 +2098,7 @@ class QuotaReserveSqlAlchemyTestCase(test.TestCase):
         context = FakeContext('test_project', 'test_class')
         quotas = dict(volumes=5, gigabytes=10 * 1024, )
         deltas = dict(volumes=-2, gigabytes=-2 * 1024, )
-        self._stub_allocated_get_all_by_project()
+        self._mock_allocated_get_all_by_project()
         result = sqa_api.quota_reserve(context, self.resources, quotas,
                                        deltas, self.expire, 0, 0)
 
@@ -2062,7 +2129,7 @@ class QuotaReserveSqlAlchemyTestCase(test.TestCase):
         context = FakeContext('test_project', 'test_class')
         quotas = dict(volumes=5, gigabytes=10 * 1024, )
         deltas = dict(volumes=2, gigabytes=2 * 1024, )
-        self._stub_allocated_get_all_by_project()
+        self._mock_allocated_get_all_by_project()
         self.assertRaises(exception.OverQuota,
                           sqa_api.quota_reserve,
                           context, self.resources, quotas,
@@ -2088,7 +2155,7 @@ class QuotaReserveSqlAlchemyTestCase(test.TestCase):
         context = FakeContext('test_project', 'test_class')
         quotas = dict(volumes=5, gigabytes=10 * 1024, )
         deltas = dict(volumes=-2, gigabytes=-2 * 1024, )
-        self._stub_allocated_get_all_by_project()
+        self._mock_allocated_get_all_by_project()
         result = sqa_api.quota_reserve(context, self.resources, quotas,
                                        deltas, self.expire, 0, 0)
 
@@ -2121,9 +2188,9 @@ class QuotaVolumeTypeReservationTestCase(test.TestCase):
         super(QuotaVolumeTypeReservationTestCase, self).setUp()
 
         self.volume_type_name = CONF.default_volume_type
-        self.volume_type = db.volume_type_create(
+        self.volume_type = db.volume_type_get_by_name(
             context.get_admin_context(),
-            dict(name=self.volume_type_name))
+            name=self.volume_type_name)
 
     @mock.patch.object(quota.QUOTAS, 'reserve')
     @mock.patch.object(quota.QUOTAS, 'add_volume_type_opts')
