@@ -17,22 +17,22 @@
 
 """Tests for the IBM FlashSystem volume driver."""
 
-import mock
+import random
+import re
+from unittest import mock
+
 from oslo_concurrency import processutils
 from oslo_utils import units
 import six
 
-import random
-import re
-
 from cinder import context
 from cinder import exception
-from cinder import test
+from cinder.tests.unit import test
 from cinder import utils
 from cinder.volume import configuration as conf
 from cinder.volume.drivers.ibm import flashsystem_fc
-from cinder.volume import utils as volume_utils
 from cinder.volume import volume_types
+from cinder.volume import volume_utils
 
 
 class FlashSystemManagementSimulator(object):
@@ -51,8 +51,15 @@ class FlashSystemManagementSimulator(object):
             # CMMVC50000 is a fake error which indicates that command has not
             # got expected results. This error represents kinds of CLI errors.
             'CMMVC50000': ('', 'CMMVC50000 The command can not be executed '
-                               'successfully.')
+                               'successfully.'),
+            'CMMVC6045E': ('', 'CMMVC6045E The action failed, as the '
+                               '-force flag was not entered.'),
+            'CMMVC6071E': ('', 'CMMVC6071E The VDisk-to-host mapping '
+                               'was not created because the VDisk is '
+                               'already mapped to a host.')
         }
+        self._multi_host_map_error = None
+        self._multi_host_map_errors = ['CMMVC6045E', 'CMMVC6071E']
 
     @staticmethod
     def _find_unused_id(d):
@@ -94,10 +101,9 @@ class FlashSystemManagementSimulator(object):
         if arg_list[0] not in ('svcinfo', 'svctask') or len(arg_list) < 2:
             raise exception.InvalidInput(reason=six.text_type(arg_list))
         ret = {'cmd': arg_list[1]}
-        arg_list.pop(0)
 
         skip = False
-        for i in range(1, len(arg_list)):
+        for i in range(2, len(arg_list)):
             if skip:
                 skip = False
                 continue
@@ -612,15 +618,14 @@ class FlashSystemManagementSimulator(object):
         if mapping_info['host'] not in self._hosts_list:
             return self._errors['CMMVC50000']
 
-        if mapping_info['vol'] in self._mappings_list:
-            return self._errors['CMMVC50000']
-
         for v in self._mappings_list.values():
+            if (v['vol'] == mapping_info['vol']) and ('force' not in kwargs):
+                return self._errors[self._multi_host_map_error or 'CMMVC50000']
+
             if ((v['host'] == mapping_info['host']) and
                     (v['lun'] == mapping_info['lun'])):
                 return self._errors['CMMVC50000']
 
-        for v in self._mappings_list.values():
             if (v['lun'] == mapping_info['lun']) and ('force' not in kwargs):
                 return self._errors['CMMVC50000']
 
@@ -662,8 +667,7 @@ class FlashSystemManagementSimulator(object):
         except exception.InvalidInput:
             return self._errors['CMMVC50000']
 
-        command = kwargs['cmd']
-        del kwargs['cmd']
+        command = kwargs.pop('cmd')
         func = getattr(self, '_cmd_' + command)
         out, err = func(**kwargs)
 
@@ -749,6 +753,13 @@ class FlashSystemDriverTestCase(test.TestCase):
             'wwpns': ['abcd000000000001', 'abcd000000000002'],
             'initiator': 'iqn.123456'}
 
+        self.alt_connector = {
+            'host': 'other',
+            'wwnns': ['0123456789fedcba', '0123456789badcfe'],
+            'wwpns': ['dcba000000000001', 'dcba000000000002'],
+            'initiator': 'iqn.654321'
+        }
+
         self.sim = FlashSystemManagementSimulator()
         self.driver = FlashSystemFakeDriver(
             configuration=conf.Configuration(None))
@@ -923,7 +934,8 @@ class FlashSystemDriverTestCase(test.TestCase):
         # case 3: _get_vdisk_map_properties raises exception
         with mock.patch.object(flashsystem_fc.FlashSystemFCDriver,
                                '_get_vdisk_map_properties') as get_properties:
-            get_properties.side_effect = exception.VolumeBackendAPIException
+            get_properties.side_effect = (
+                exception.VolumeBackendAPIException(data=''))
             self.assertRaises(exception.VolumeBackendAPIException,
                               self.driver.initialize_connection,
                               vol1, self.connector)
@@ -1029,12 +1041,15 @@ class FlashSystemDriverTestCase(test.TestCase):
         vol2 = self._generate_vol_info(None)
         self.driver.create_cloned_volume(vol2, vol1)
 
-        # case 2: when size does not match
+        # case 2: destination larger than source
         vol1 = self._generate_vol_info(None, vol_size=10)
         vol2 = self._generate_vol_info(None, vol_size=20)
+        self.driver.create_cloned_volume(vol2, vol1)
+
+        # case 3: destination smaller than source
         self.assertRaises(exception.VolumeDriverException,
                           self.driver.create_cloned_volume,
-                          vol2, vol1)
+                          vol1, vol2)
 
     def test_flashsystem_get_volume_stats(self):
         # case 1: good path
@@ -1077,7 +1092,8 @@ class FlashSystemDriverTestCase(test.TestCase):
 
         # case 3: _copy_vdisk_data raises exception
         self.driver.create_volume(vol1)
-        _copy_vdisk_data.side_effect = exception.VolumeBackendAPIException
+        _copy_vdisk_data.side_effect = (
+            exception.VolumeBackendAPIException(data=''))
         self.assertRaises(
             exception.VolumeBackendAPIException,
             self.driver._create_and_copy_vdisk_data,
@@ -1125,7 +1141,7 @@ class FlashSystemDriverTestCase(test.TestCase):
         self.driver.terminate_connection(vol2, connector)
 
         # case 3: no mapped before copy, raise exception when scan
-        _scan_device.side_effect = exception.VolumeBackendAPIException
+        _scan_device.side_effect = exception.VolumeBackendAPIException(data='')
         self.assertRaises(
             exception.VolumeBackendAPIException,
             self.driver._copy_vdisk_data,
@@ -1136,7 +1152,7 @@ class FlashSystemDriverTestCase(test.TestCase):
         self.assertFalse(v2_mapped)
 
         # case 4: no mapped before copy, raise exception when copy
-        copy_volume.side_effect = exception.VolumeBackendAPIException
+        copy_volume.side_effect = exception.VolumeBackendAPIException(data='')
         self.assertRaises(
             exception.VolumeBackendAPIException,
             self.driver._copy_vdisk_data,
@@ -1165,7 +1181,7 @@ class FlashSystemDriverTestCase(test.TestCase):
 
         # case 3: host name is neither unicode nor string
         conn = {'host': 12345}
-        self.assertRaises(exception.NoValidHost,
+        self.assertRaises(exception.NoValidBackend,
                           self.driver._connector_to_hostname_prefix,
                           conn)
 
@@ -1280,6 +1296,25 @@ class FlashSystemDriverTestCase(test.TestCase):
             1,
             self.driver._map_vdisk_to_host(vol1['name'], self.connector))
 
+        # case 4: multi-host mapping, good path
+        for error in self.sim._multi_host_map_errors:
+            self.sim._multi_host_map_error = error
+            self.assertEqual(
+                1,
+                self.driver._map_vdisk_to_host(
+                    vol1['name'], self.alt_connector
+                )
+            )
+            self.driver._unmap_vdisk_from_host(
+                vol1['name'], self.alt_connector
+            )
+        self.sim._multi_host_map_error = None
+
+        # case 5: multi-host mapping, bad path
+        self.assertRaises(
+            exception.VolumeBackendAPIException,
+            self.driver._map_vdisk_to_host, vol1['name'], self.alt_connector)
+
         # clean environment
         self.driver._unmap_vdisk_from_host(vol1['name'], self.connector)
         self.driver._unmap_vdisk_from_host(vol2['name'], self.connector)
