@@ -13,9 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
 
 from lxml import etree
-import mock
 from six.moves import urllib
 
 from cinder.tests.unit.volume.drivers.netapp.dataontap import fakes as fake
@@ -277,81 +277,6 @@ SNAPSHOT_INFO_FOR_PRESENT_BUSY_SNAPSHOT_CMODE = etree.XML("""
     'vol_name': fake.SNAPSHOT['volume_id'],
 })
 
-SNAPSHOT_INFO_FOR_PRESENT_NOT_BUSY_SNAPSHOT_7MODE = etree.XML("""
-    <results status="passed">
-    <snapshots>
-      <snapshot-info>
-        <name>%(snapshot_name)s</name>
-        <busy>False</busy>
-        <volume>%(vol_name)s</volume>
-        <snapshot-instance-uuid>abcd-ef01-2345-6789</snapshot-instance-uuid>
-      </snapshot-info>
-    </snapshots>
-    </results>
-""" % {
-    'snapshot_name': fake.SNAPSHOT['name'],
-    'vol_name': fake.SNAPSHOT['volume_id'],
-})
-
-SNAPSHOT_INFO_MARKED_FOR_DELETE_SNAPSHOT_7MODE = etree.XML("""
-    <results status="passed">
-    <snapshots>
-      <snapshot-info>
-        <name>deleted_cinder_%(snapshot_name)s</name>
-        <busy>False</busy>
-        <volume>%(vol_name)s</volume>
-        <snapshot-instance-uuid>abcd-ef01-2345-6789</snapshot-instance-uuid>
-      </snapshot-info>
-    </snapshots>
-    </results>
-""" % {
-    'snapshot_name': fake.SNAPSHOT['name'],
-    'vol_name': fake.SNAPSHOT['volume_id'],
-})
-
-SNAPSHOT_INFO_MARKED_FOR_DELETE_SNAPSHOT_7MODE_BUSY = etree.XML("""
-    <results status="passed">
-    <snapshots>
-      <snapshot-info>
-        <name>deleted_cinder_busy_snapshot</name>
-        <busy>True</busy>
-        <volume>%(vol_name)s</volume>
-        <snapshot-instance-uuid>abcd-ef01-2345-6789</snapshot-instance-uuid>
-      </snapshot-info>
-    </snapshots>
-    </results>
-""" % {
-    'snapshot_name': fake.SNAPSHOT['name'],
-    'vol_name': fake.SNAPSHOT['volume_id'],
-})
-
-SNAPSHOT_INFO_FOR_PRESENT_BUSY_SNAPSHOT_7MODE = etree.XML("""
-    <results status="passed">
-    <snapshots>
-      <snapshot-info>
-        <name>%(snapshot_name)s</name>
-        <busy>True</busy>
-        <volume>%(vol_name)s</volume>
-      </snapshot-info>
-    </snapshots>
-    </results>
-""" % {
-    'snapshot_name': fake.SNAPSHOT['name'],
-    'vol_name': fake.SNAPSHOT['volume_id'],
-})
-
-SNAPSHOT_NOT_PRESENT_7MODE = etree.XML("""
-    <results status="passed">
-    <snapshots>
-      <snapshot-info>
-        <name>NOT_THE_RIGHT_SNAPSHOT</name>
-        <busy>false</busy>
-        <volume>%(vol_name)s</volume>
-      </snapshot-info>
-    </snapshots>
-    </results>
-""" % {'vol_name': fake.SNAPSHOT['volume_id']})
-
 NODE_NAME = 'fake_node1'
 NODE_NAMES = ('fake_node1', 'fake_node2')
 VOLUME_AGGREGATE_NAME = 'fake_aggr1'
@@ -834,6 +759,8 @@ SIS_GET_ITER_SSC_RESPONSE = etree.XML("""
       <sis-status-info>
         <is-compression-enabled>false</is-compression-enabled>
         <state>enabled</state>
+        <logical-data-size>211106232532992</logical-data-size>
+        <logical-data-limit>703687441776640</logical-data-limit>
       </sis-status-info>
     </attributes-list>
     <num-records>1</num-records>
@@ -843,8 +770,91 @@ SIS_GET_ITER_SSC_RESPONSE = etree.XML("""
 VOLUME_DEDUPE_INFO_SSC = {
     'compression': False,
     'dedupe': True,
+    'logical-data-size': 211106232532992,
+    'logical-data-limit': 703687441776640,
 }
 
+SIS_GET_ITER_SSC_NO_LOGICAL_DATA_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <sis-status-info>
+        <is-compression-enabled>false</is-compression-enabled>
+        <state>disabled</state>
+      </sis-status-info>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""")
+
+VOLUME_DEDUPE_INFO_SSC_NO_LOGICAL_DATA = {
+    'compression': False,
+    'dedupe': False,
+    'logical-data-size': 0,
+    'logical-data-limit': 1,
+}
+
+CLONE_SPLIT_STATUS_RESPONSE = etree.XML("""
+  <results status="passed">
+    <clone-split-info>
+      <unsplit-clone-count>1234</unsplit-clone-count>
+      <unsplit-size>316659348799488</unsplit-size>
+    </clone-split-info>
+  </results>
+""")
+
+VOLUME_CLONE_SPLIT_STATUS = {
+    'unsplit-size': 316659348799488,
+    'unsplit-clone-count': 1234,
+}
+
+CLONE_SPLIT_STATUS_NO_DATA_RESPONSE = etree.XML("""
+  <results status="passed">
+    <clone-split-info>
+    </clone-split-info>
+  </results>
+""")
+
+VOLUME_GET_ITER_ENCRYPTION_SSC_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <volume-attributes>
+        <encrypt>true</encrypt>
+        <volume-id-attributes>
+          <containing-aggregate-name>%(aggr)s</containing-aggregate-name>
+          <junction-path>/%(volume)s</junction-path>
+          <name>%(volume)s</name>
+          <owning-vserver-name>%(vserver)s</owning-vserver-name>
+          <type>rw</type>
+        </volume-id-attributes>
+        <volume-mirror-attributes>
+          <is-data-protection-mirror>false</is-data-protection-mirror>
+          <is-replica-volume>false</is-replica-volume>
+        </volume-mirror-attributes>
+        <volume-qos-attributes>
+          <policy-group-name>fake_qos_policy_group_name</policy-group-name>
+        </volume-qos-attributes>
+        <volume-space-attributes>
+          <is-space-guarantee-enabled>true</is-space-guarantee-enabled>
+          <space-guarantee>none</space-guarantee>
+          <percentage-snapshot-reserve>5</percentage-snapshot-reserve>
+          <size>12345</size>
+        </volume-space-attributes>
+        <volume-snapshot-attributes>
+          <snapshot-policy>default</snapshot-policy>
+        </volume-snapshot-attributes>
+        <volume-language-attributes>
+          <language-code>en_US</language-code>
+        </volume-language-attributes>
+      </volume-attributes>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""" % {
+    'aggr': VOLUME_AGGREGATE_NAMES[0],
+    'volume': VOLUME_NAMES[0],
+    'vserver': VOLUME_VSERVER_NAME,
+})
+
 STORAGE_DISK_GET_ITER_RESPONSE_PAGE_1 = etree.XML("""
   <results status="passed">
     <attributes-list>
@@ -1093,22 +1103,6 @@ PERF_OBJECT_GET_INSTANCES_SYSTEM_RESPONSE_CMODE = etree.XML("""
   </results>
 """ % {'node1': NODE_NAMES[0], 'node2': NODE_NAMES[1]})
 
-PERF_OBJECT_GET_INSTANCES_SYSTEM_RESPONSE_7MODE = etree.XML("""
-  <results status="passed">
-    <timestamp>1454146292</timestamp>
-    <instances>
-      <instance-data>
-        <name>system</name>
-        <counters>
-          <counter-data>
-            <name>avg_processor_busy</name>
-            <value>13215732322</value>
-          </counter-data>
-        </counters>
-      </instance-data>
-    </instances>
-  </results>""")
-
 PERF_OBJECT_INSTANCE_LIST_INFO_ITER_RESPONSE = etree.XML("""
   <results status="passed">
     <attributes-list>
@@ -1172,6 +1166,8 @@ CLUSTER_ADDRESS_1 = 'fake_cluster_address'
 CLUSTER_ADDRESS_2 = 'fake_cluster_address_2'
 VSERVER_NAME = 'fake_vserver'
 VSERVER_NAME_2 = 'fake_vserver_2'
+ADMIN_VSERVER_NAME = 'fake_admin_vserver'
+NODE_VSERVER_NAME = 'fake_node_vserver'
 SM_SOURCE_VSERVER = 'fake_source_vserver'
 SM_SOURCE_VOLUME = 'fake_source_volume'
 SM_DEST_VSERVER = 'fake_destination_vserver'
@@ -1301,3 +1297,26 @@ SNAPMIRROR_INITIALIZE_RESULT = etree.XML("""
     <result-status>succeeded</result-status>
   </results>
 """)
+
+VSERVER_DATA_LIST_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <vserver-info>
+        <vserver-name>%(vserver)s</vserver-name>
+        <vserver-type>data</vserver-type>
+      </vserver-info>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""" % {'vserver': VSERVER_NAME})
+
+SYSTEM_NODE_GET_ITER_RESPONSE = etree.XML("""
+  <results status="passed">
+    <attributes-list>
+      <node-details-info>
+        <node>%s</node>
+      </node-details-info>
+    </attributes-list>
+    <num-records>1</num-records>
+  </results>
+""" % NODE_NAME)
