@@ -15,19 +15,20 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-"""
-Tests for NetApp API layer
-"""
+"""Tests for NetApp API layer"""
+
+from unittest import mock
+
 import ddt
 from lxml import etree
-import mock
+from oslo_utils import netutils
 import paramiko
 import six
 from six.moves import urllib
 
 from cinder import exception
 from cinder.i18n import _
-from cinder import test
+from cinder.tests.unit import test
 from cinder.tests.unit.volume.drivers.netapp.dataontap.client import (
     fakes as zapi_fakes)
 from cinder.volume.drivers.netapp.dataontap.client import api as netapp_api
@@ -237,6 +238,25 @@ class NetAppApiServerTests(test.TestCase):
 
         self.root.send_http_request(na_element)
 
+    @ddt.data('192.168.1.0', '127.0.0.1', '0.0.0.0',
+              '::ffff:8', 'fdf8:f53b:82e4::53', '2001::1',
+              'fe80::200::abcd', '2001:0000:4136:e378:8000:63bf:3fff:fdd2')
+    def test__get_url(self, host):
+        port = '80'
+        root = netapp_api.NaServer(host, port=port)
+
+        protocol = root.TRANSPORT_TYPE_HTTP
+        url = root.URL_FILER
+
+        if netutils.is_valid_ipv6(host):
+            host = netutils.escape_ipv6(host)
+
+        result = '%s://%s:%s/%s' % (protocol, host, port, url)
+
+        url = root._get_url()
+
+        self.assertEqual(result, url)
+
 
 class NetAppApiElementTransTests(test.TestCase):
     """Test case for NetApp API element translations."""
@@ -331,11 +351,11 @@ class NetAppApiElementTransTests(test.TestCase):
         root = netapp_api.NaElement('root')
         root['l'] = ['l1', 'l2']
         root['t'] = ('t1', 't2')
-        l = root.get_child_by_name('l')
-        self.assertIsInstance(l, netapp_api.NaElement)
+        l_element = root.get_child_by_name('l')
+        self.assertIsInstance(l_element, netapp_api.NaElement)
         t = root.get_child_by_name('t')
         self.assertIsInstance(t, netapp_api.NaElement)
-        for le in l.get_children():
+        for le in l_element.get_children():
             self.assertIn(le.get_name(), ['l1', 'l2'])
         for te in t.get_children():
             self.assertIn(te.get_name(), ['t1', 't2'])
