@@ -12,22 +12,20 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-"""
-Mock unit tests for the NetApp cmode nfs storage driver
-"""
+"""Mock unit tests for the NetApp cmode nfs storage driver."""
 
 import hashlib
+from unittest import mock
 import uuid
 
 import ddt
-import mock
 from os_brick.remotefs import remotefs as remotefs_brick
 from oslo_utils import units
 
 from cinder import exception
 from cinder.image import image_utils
 from cinder.objects import fields
-from cinder import test
+from cinder.tests.unit import test
 from cinder.tests.unit.volume.drivers.netapp.dataontap import fakes as fake
 from cinder.tests.unit.volume.drivers.netapp.dataontap.utils import fakes as \
     fake_ssc
@@ -44,7 +42,7 @@ from cinder.volume.drivers.netapp.dataontap.utils import loopingcalls
 from cinder.volume.drivers.netapp.dataontap.utils import utils as dot_utils
 from cinder.volume.drivers.netapp import utils as na_utils
 from cinder.volume.drivers import nfs
-from cinder.volume import utils as volume_utils
+from cinder.volume import volume_utils
 
 
 @ddt.ddt
@@ -307,7 +305,7 @@ class NetAppCmodeNfsDriverTestCase(test.TestCase):
             self.driver.zapi_client, 'get_operational_lif_addresses',
             return_value=[fake.SHARE_IP])
         mock_resolve_hostname = self.mock_object(
-            na_utils, 'resolve_hostname', return_value=fake.SHARE_IP)
+            volume_utils, 'resolve_hostname', return_value=fake.SHARE_IP)
         mock_get_flexvol = self.mock_object(
             self.driver.zapi_client, 'get_flexvol',
             return_value={'name': fake.NETAPP_VOLUME})
@@ -330,7 +328,7 @@ class NetAppCmodeNfsDriverTestCase(test.TestCase):
         self.mock_object(self.driver.zapi_client,
                          'get_operational_lif_addresses',
                          return_value=[])
-        self.mock_object(na_utils,
+        self.mock_object(volume_utils,
                          'resolve_hostname',
                          return_value=fake.SHARE_IP)
 
@@ -344,7 +342,7 @@ class NetAppCmodeNfsDriverTestCase(test.TestCase):
         self.mock_object(self.driver.zapi_client,
                          'get_operational_lif_addresses',
                          return_value=[fake.SHARE_IP])
-        self.mock_object(na_utils,
+        self.mock_object(volume_utils,
                          'resolve_hostname',
                          return_value=fake.SHARE_IP)
         side_effect = exception.VolumeBackendAPIException(data='fake_data')
@@ -517,7 +515,7 @@ class NetAppCmodeNfsDriverTestCase(test.TestCase):
         self.mock_object(
             self.driver.zapi_client,
             'mark_qos_policy_group_for_deletion',
-            side_effect=exception.NetAppDriverException)
+            side_effect=na_utils.NetAppDriverException)
 
         self.driver.delete_volume(fake_volume)
 
@@ -637,7 +635,7 @@ class NetAppCmodeNfsDriverTestCase(test.TestCase):
         mock_provision_qos.assert_has_calls([
             mock.call(fake.QOS_POLICY_GROUP_INFO)])
         mock_set_policy.assert_has_calls([
-            mock.call(fake.NFS_VOLUME, fake.QOS_POLICY_GROUP_INFO)])
+            mock.call(fake.NFS_VOLUME, fake.QOS_POLICY_GROUP_INFO, False)])
         self.assertEqual(0, mock_error_log.call_count)
         self.assertEqual(0, mock_debug_log.call_count)
         self.assertEqual(0, mock_cleanup.call_count)
@@ -665,7 +663,7 @@ class NetAppCmodeNfsDriverTestCase(test.TestCase):
         mock_provision_qos.assert_has_calls([
             mock.call(fake.QOS_POLICY_GROUP_INFO)])
         mock_set_policy.assert_has_calls([
-            mock.call(fake.NFS_VOLUME, fake.QOS_POLICY_GROUP_INFO)])
+            mock.call(fake.NFS_VOLUME, fake.QOS_POLICY_GROUP_INFO, False)])
         self.assertEqual(1, mock_error_log.call_count)
         self.assertEqual(1, mock_debug_log.call_count)
         mock_cleanup.assert_has_calls([
@@ -710,8 +708,8 @@ class NetAppCmodeNfsDriverTestCase(test.TestCase):
 
         mock_file_assign_qos = self.driver.zapi_client.file_assign_qos
 
-        self.driver._set_qos_policy_group_on_volume(fake.NFS_VOLUME,
-                                                    fake.QOS_POLICY_GROUP_INFO)
+        self.driver._set_qos_policy_group_on_volume(
+            fake.NFS_VOLUME, fake.QOS_POLICY_GROUP_INFO, False)
 
         mock_get_name_from_info.assert_has_calls([
             mock.call(fake.QOS_POLICY_GROUP_INFO)])
@@ -721,7 +719,7 @@ class NetAppCmodeNfsDriverTestCase(test.TestCase):
             mock.call(fake.VSERVER_NAME, fake.EXPORT_PATH)])
         mock_file_assign_qos.assert_has_calls([
             mock.call(fake.FLEXVOL, fake.QOS_POLICY_GROUP_NAME,
-                      fake.NFS_VOLUME['name'])])
+                      False, fake.NFS_VOLUME['name'])])
 
     def test_set_qos_policy_group_on_volume_no_info(self):
 
@@ -736,7 +734,7 @@ class NetAppCmodeNfsDriverTestCase(test.TestCase):
         mock_file_assign_qos = self.driver.zapi_client.file_assign_qos
 
         self.driver._set_qos_policy_group_on_volume(fake.NFS_VOLUME,
-                                                    None)
+                                                    None, False)
 
         self.assertEqual(0, mock_get_name_from_info.call_count)
         self.assertEqual(0, mock_extract_host.call_count)
@@ -756,8 +754,8 @@ class NetAppCmodeNfsDriverTestCase(test.TestCase):
 
         mock_file_assign_qos = self.driver.zapi_client.file_assign_qos
 
-        self.driver._set_qos_policy_group_on_volume(fake.NFS_VOLUME,
-                                                    fake.QOS_POLICY_GROUP_INFO)
+        self.driver._set_qos_policy_group_on_volume(
+            fake.NFS_VOLUME, fake.QOS_POLICY_GROUP_INFO, False)
 
         mock_get_name_from_info.assert_has_calls([
             mock.call(fake.QOS_POLICY_GROUP_INFO)])
@@ -875,8 +873,10 @@ class NetAppCmodeNfsDriverTestCase(test.TestCase):
     @mock.patch.object(image_utils, 'convert_image')
     @mock.patch.object(image_utils, 'qemu_img_info')
     @mock.patch('os.path.exists')
+    @mock.patch('cinder.privsep.path')
     def test_copy_from_img_service_qcow2_copyoffload_workflow_success(
-            self, mock_exists, mock_qemu_img_info, mock_cvrt_image):
+            self, mock_touch, mock_exists, mock_qemu_img_info,
+            mock_cvrt_image):
         drv = self.driver
         cinder_mount_point_base = '/opt/stack/data/cinder/mnt/'
         # To get the cinder mount point directory, we use:
@@ -933,11 +933,10 @@ class NetAppCmodeNfsDriverTestCase(test.TestCase):
                     '203.0.113.122', '/openstack/glance-flexvol1',
                     destination_copied_file, run_as_root=False,
                     check_exit_code=0
-                ),
-                mock.call('touch', cinder_mount_point, run_as_root=False)
+                )
             ]
         )
-        self.assertEqual(2, drv._execute.call_count)
+        self.assertEqual(1, drv._execute.call_count)
         self.assertEqual(2, drv._delete_file_at_path.call_count)
         self.assertEqual(1, drv._clone_file_dst_exists.call_count)
 
@@ -1365,7 +1364,7 @@ class NetAppCmodeNfsDriverTestCase(test.TestCase):
         """This tests executes a method in the DataMotionMixin."""
         self.driver.backend_name = 'dev0'
         self.mock_object(data_motion.DataMotionMixin, '_complete_failover',
-                         side_effect=exception.NetAppDriverException)
+                         side_effect=na_utils.NetAppDriverException)
         self.mock_object(data_motion.DataMotionMixin,
                          'get_replication_backend_names',
                          return_value=['dev1', 'dev2'])
@@ -1547,7 +1546,7 @@ class NetAppCmodeNfsDriverTestCase(test.TestCase):
         self.mock_object(self.driver.zapi_client, 'create_cg_snapshot',
                          side_effect=netapp_api.NaApiError)
 
-        self.assertRaises(exception.NetAppDriverException,
+        self.assertRaises(na_utils.NetAppDriverException,
                           self.driver.create_group_snapshot,
                           fake.VG_CONTEXT,
                           fake.VOLUME_GROUP,
