@@ -13,23 +13,29 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 """Unit tests for kaminario driver."""
-import mock
-from oslo_utils import units
+import re
 import time
+from unittest import mock
+
+import ddt
+from oslo_config import cfg
+from oslo_utils import units
 
 from cinder import context
 from cinder import exception
 from cinder import objects
 from cinder.objects import fields
-from cinder import test
 from cinder.tests.unit import fake_snapshot
 from cinder.tests.unit import fake_volume
+from cinder.tests.unit import test
 from cinder import utils
 from cinder.volume import configuration
 from cinder.volume.drivers.kaminario import kaminario_common
 from cinder.volume.drivers.kaminario import kaminario_fc
 from cinder.volume.drivers.kaminario import kaminario_iscsi
-from cinder.volume import utils as vol_utils
+from cinder.volume import volume_utils
+
+CONF = cfg.CONF
 
 CONNECTOR = {'initiator': 'iqn.1993-08.org.debian:01:12aa12aa12aa',
              'ip': '192.168.2.5', 'platform': 'x86_64', 'host': 'test-k2',
@@ -45,12 +51,13 @@ class FakeK2Obj(object):
 
 class FakeSaveObject(FakeK2Obj):
     def __init__(self, *args, **kwargs):
+        item = kwargs.pop('item', 1)
         self.ntype = kwargs.get('ntype')
-        self.ip_address = '10.0.0.1'
+        self.ip_address = '10.0.0.%s' % item
         self.iscsi_qualified_target_name = "xyztlnxyz"
         self.snapshot = FakeK2Obj()
         self.name = 'test'
-        self.pwwn = '50024f4053300300'
+        self.pwwn = '50024f405330030%s' % item
         self.volume_group = self
         self.is_dedup = True
         self.size = units.Mi
@@ -60,6 +67,7 @@ class FakeSaveObject(FakeK2Obj):
         self.current_role = 'target'
         self.current_snapshot_progress = 100
         self.current_snapshot_id = None
+        self.wan_port = None
 
     def refresh(self):
         return
@@ -73,15 +81,15 @@ class FakeSaveObject(FakeK2Obj):
 
 class FakeSaveObjectExp(FakeSaveObject):
     def save(self):
-        raise exception.KaminarioCinderDriverException("test")
+        raise kaminario_common.KaminarioCinderDriverException("test")
 
     def delete(self):
-        raise exception.KaminarioCinderDriverException("test")
+        raise kaminario_common.KaminarioCinderDriverException("test")
 
 
 class FakeSearchObject(object):
-    hits = [FakeSaveObject()]
-    total = 1
+    hits = [FakeSaveObject(item=1), FakeSaveObject(item=2)]
+    total = 2
 
     def __init__(self, *args):
         if args and "mappings" in args[0]:
@@ -116,25 +124,29 @@ class Replication(object):
     rpo = 500
 
 
-class TestKaminarioISCSI(test.TestCase):
+@ddt.ddt
+class TestKaminarioCommon(test.TestCase):
     driver = None
     conf = None
 
     def setUp(self):
         self._setup_config()
         self._setup_driver()
-        super(TestKaminarioISCSI, self).setUp()
+        super(TestKaminarioCommon, self).setUp()
         self.context = context.get_admin_context()
         self.vol = fake_volume.fake_volume_obj(self.context)
         self.vol.volume_type = fake_volume.fake_volume_type_obj(self.context)
         self.vol.volume_type.extra_specs = {'foo': None}
         self.snap = fake_snapshot.fake_snapshot_obj(self.context)
         self.snap.volume = self.vol
+        self.patch('eventlet.sleep')
 
     def _setup_config(self):
         self.conf = mock.Mock(spec=configuration.Configuration)
         self.conf.kaminario_dedup_type_name = "dedup"
         self.conf.volume_dd_blocksize = 2
+        self.conf.disable_discovery = False
+        self.conf.unique_fqdn_network = True
 
     def _setup_driver(self):
         self.driver = (kaminario_iscsi.
@@ -151,7 +163,7 @@ class TestKaminarioISCSI(test.TestCase):
     def test_create_volume_with_exception(self):
         """Test create_volume_with_exception."""
         self.driver.client = FakeKrestException()
-        self.assertRaises(exception.KaminarioCinderDriverException,
+        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                           self.driver.create_volume, self.vol)
 
     def test_delete_volume(self):
@@ -162,7 +174,7 @@ class TestKaminarioISCSI(test.TestCase):
     def test_delete_volume_with_exception(self):
         """Test delete_volume_with_exception."""
         self.driver.client = FakeKrestException()
-        self.assertRaises(exception.KaminarioCinderDriverException,
+        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                           self.driver.delete_volume, self.vol)
 
     def test_create_snapshot(self):
@@ -181,7 +193,7 @@ class TestKaminarioISCSI(test.TestCase):
     def test_create_snapshot_with_exception(self):
         """Test create_snapshot_with_exception."""
         self.driver.client = FakeKrestException()
-        self.assertRaises(exception.KaminarioCinderDriverException,
+        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                           self.driver.create_snapshot, self.snap)
 
     def test_delete_snapshot(self):
@@ -192,11 +204,11 @@ class TestKaminarioISCSI(test.TestCase):
     def test_delete_snapshot_with_exception(self):
         """Test delete_snapshot_with_exception."""
         self.driver.client = FakeKrestException()
-        self.assertRaises(exception.KaminarioCinderDriverException,
+        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                           self.driver.delete_snapshot, self.snap)
 
     @mock.patch.object(utils, 'brick_get_connector_properties')
-    @mock.patch.object(vol_utils, 'copy_volume')
+    @mock.patch.object(volume_utils, 'copy_volume')
     def test_create_volume_from_snapshot(self, mock_copy_volume,
                                          mock_brick_get):
         """Test create_volume_from_snapshot."""
@@ -207,19 +219,19 @@ class TestKaminarioISCSI(test.TestCase):
         self.assertIsNone(result)
 
     @mock.patch.object(utils, 'brick_get_connector_properties')
-    @mock.patch.object(vol_utils, 'copy_volume')
+    @mock.patch.object(volume_utils, 'copy_volume')
     def test_create_volume_from_snapshot_with_exception(self, mock_copy_volume,
                                                         mock_brick_get):
         """Test create_volume_from_snapshot_with_exception."""
         mock_brick_get.return_value = CONNECTOR
         mock_copy_volume.return_value = None
         self.driver.client = FakeKrestException()
-        self.assertRaises(exception.KaminarioCinderDriverException,
+        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                           self.driver.create_volume_from_snapshot, self.vol,
                           self.snap)
 
     @mock.patch.object(utils, 'brick_get_connector_properties')
-    @mock.patch.object(vol_utils, 'copy_volume')
+    @mock.patch.object(volume_utils, 'copy_volume')
     def test_create_cloned_volume(self, mock_copy_volume, mock_brick_get):
         """Test create_cloned_volume."""
         mock_brick_get.return_value = CONNECTOR
@@ -229,7 +241,7 @@ class TestKaminarioISCSI(test.TestCase):
         self.assertIsNone(result)
 
     @mock.patch.object(utils, 'brick_get_connector_properties')
-    @mock.patch.object(vol_utils, 'copy_volume')
+    @mock.patch.object(volume_utils, 'copy_volume')
     def test_create_cloned_volume_with_exception(self, mock_copy_volume,
                                                  mock_brick_get):
         """Test create_cloned_volume_with_exception."""
@@ -237,7 +249,7 @@ class TestKaminarioISCSI(test.TestCase):
         mock_copy_volume.return_value = None
         self.driver.terminate_connection = mock.Mock()
         self.driver.client = FakeKrestException()
-        self.assertRaises(exception.KaminarioCinderDriverException,
+        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                           self.driver.create_cloned_volume, self.vol, self.vol)
 
     def test_extend_volume(self):
@@ -250,27 +262,16 @@ class TestKaminarioISCSI(test.TestCase):
         """Test extend_volume_with_exception."""
         self.driver.client = FakeKrestException()
         new_size = 256
-        self.assertRaises(exception.KaminarioCinderDriverException,
+        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                           self.driver.extend_volume, self.vol, new_size)
 
-    def test_initialize_connection(self):
-        """Test initialize_connection."""
-        conn_info = self.driver.initialize_connection(self.vol, CONNECTOR)
-        self.assertIn('data', conn_info)
-        self.assertIn('target_iqn', conn_info['data'])
-
     def test_initialize_connection_with_exception(self):
         """Test initialize_connection_with_exception."""
         self.driver.client = FakeKrestException()
-        self.assertRaises(exception.KaminarioCinderDriverException,
+        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                           self.driver.initialize_connection, self.vol,
                           CONNECTOR)
 
-    def test_terminate_connection(self):
-        """Test terminate_connection."""
-        result = self.driver.terminate_connection(self.vol, CONNECTOR)
-        self.assertIsNone(result)
-
     def test_get_lun_number(self):
         """Test _get_lun_number."""
         host, host_rs, host_name = self.driver._get_host_object(CONNECTOR)
@@ -286,20 +287,15 @@ class TestKaminarioISCSI(test.TestCase):
         """Test _get_host_object."""
         host, host_rs, host_name = self.driver._get_host_object(CONNECTOR)
         self.assertEqual(548, host.id)
-        self.assertEqual(1, host_rs.total)
+        self.assertEqual(2, host_rs.total)
         self.assertEqual('test-k2', host_name)
 
-    def test_get_target_info(self):
-        """Test get_target_info."""
-        iscsi_portal, target_iqn = self.driver.get_target_info(self.vol)
-        self.assertEqual('10.0.0.1:3260', iscsi_portal)
-        self.assertEqual('xyztlnxyz', target_iqn)
-
     def test_k2_initialize_connection(self):
         """Test k2_initialize_connection."""
         result = self.driver.k2_initialize_connection(self.vol, CONNECTOR)
         self.assertEqual(548, result)
 
+    @mock.patch.object(FakeSearchObject, 'total', 1)
     def test_manage_existing(self):
         """Test manage_existing."""
         self.driver._get_replica_status = mock.Mock(return_value=False)
@@ -312,6 +308,12 @@ class TestKaminarioISCSI(test.TestCase):
                           self.driver.manage_existing, self.vol,
                           {'source-name': 'test'})
 
+    def test_manage_vg_volumes(self):
+        self.driver.nvol = 2
+        self.assertRaises(exception.ManageExistingInvalidReference,
+                          self.driver.manage_existing, self.vol,
+                          {'source-name': 'test'})
+
     def test_manage_existing_get_size(self):
         """Test manage_existing_get_size."""
         self.driver.client.search().hits[0].size = units.Mi
@@ -356,7 +358,7 @@ class TestKaminarioISCSI(test.TestCase):
         rep = Replication()
         self.driver.replica = rep
         self.driver.client = FakeKrestException()
-        self.assertRaises(exception.KaminarioCinderDriverException,
+        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                           self.driver._create_volume_replica, self.vol,
                           vg, vg, rep.rpo)
 
@@ -392,14 +394,16 @@ class TestKaminarioISCSI(test.TestCase):
         self.driver.target = FakeKrest()
         self.driver.target.search().total = 1
         self.driver.client.search().total = 1
-        backend_ip, res_volumes = self.driver.failover_host(None, volumes)
+        backend_ip, res_volumes, __ = self.driver.failover_host(
+            None, volumes, [])
         self.assertEqual('10.0.0.1', backend_ip)
         status = res_volumes[0]['updates']['replication_status']
         self.assertEqual(fields.ReplicationStatus.FAILED_OVER, status)
         # different backend ip
         self.driver.configuration.san_ip = '10.0.0.2'
         self.driver.client.search().hits[0].state = 'in_sync'
-        backend_ip, res_volumes = self.driver.failover_host(None, volumes)
+        backend_ip, res_volumes, __ = self.driver.failover_host(
+            None, volumes, [])
         self.assertEqual('10.0.0.2', backend_ip)
         status = res_volumes[0]['updates']['replication_status']
         self.assertEqual(fields.ReplicationStatus.DISABLED, status)
@@ -425,7 +429,7 @@ class TestKaminarioISCSI(test.TestCase):
         self.driver.replica = Replication()
         self.driver.target = FakeKrestException()
         self.driver._check_for_status = mock.Mock()
-        self.assertRaises(exception.KaminarioCinderDriverException,
+        self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                           self.driver._delete_volume_replica, self.vol,
                           'test', 'test')
 
@@ -519,8 +523,83 @@ class TestKaminarioISCSI(test.TestCase):
                                                              'test')
         self.assertIsNone(result)
 
+    def test_get_initiator_host_name(self):
+        result = self.driver.get_initiator_host_name(CONNECTOR)
+        self.assertEqual(CONNECTOR['host'], result)
 
-class TestKaminarioFC(TestKaminarioISCSI):
+    @ddt.data(True, False)
+    def test_get_initiator_host_name_unique(self, in_shared):
+        cfg = self._set_unique_fqdn_override(False, in_shared)
+        self.mock_object(self.driver, 'configuration', cfg)
+        result = self.driver.get_initiator_host_name(CONNECTOR)
+        expected = re.sub('[:.]', '_', CONNECTOR['initiator'][::-1][:32])
+        self.assertEqual(expected, result)
+
+
+@ddt.ddt
+class TestKaminarioISCSI(TestKaminarioCommon):
+    def test_get_target_info(self):
+        """Test get_target_info."""
+        iscsi_portals, target_iqns = self.driver.get_target_info(self.vol)
+        self.assertEqual(['10.0.0.1:3260', '10.0.0.2:3260'],
+                         iscsi_portals)
+        self.assertEqual(['xyztlnxyz', 'xyztlnxyz'],
+                         target_iqns)
+
+    @ddt.data(True, False)
+    def test_initialize_connection(self, multipath):
+        """Test initialize_connection."""
+        connector = CONNECTOR.copy()
+        connector['multipath'] = multipath
+        self.driver.configuration.disable_discovery = False
+
+        conn_info = self.driver.initialize_connection(self.vol, CONNECTOR)
+        expected = {
+            'data': {
+                'target_discovered': True,
+                'target_iqn': 'xyztlnxyz',
+                'target_lun': 548,
+                'target_portal': '10.0.0.1:3260',
+            },
+            'driver_volume_type': 'iscsi',
+        }
+        self.assertEqual(expected, conn_info)
+
+    def test_initialize_connection_multipath(self):
+        """Test initialize_connection with multipath."""
+        connector = CONNECTOR.copy()
+        connector['multipath'] = True
+        self.driver.configuration.disable_discovery = True
+
+        conn_info = self.driver.initialize_connection(self.vol, connector)
+
+        expected = {
+            'data': {
+                'target_discovered': True,
+                'target_iqn': 'xyztlnxyz',
+                'target_iqns': ['xyztlnxyz', 'xyztlnxyz'],
+                'target_lun': 548,
+                'target_luns': [548, 548],
+                'target_portal': '10.0.0.1:3260',
+                'target_portals': ['10.0.0.1:3260', '10.0.0.2:3260'],
+            },
+            'driver_volume_type': 'iscsi',
+        }
+        self.assertEqual(expected, conn_info)
+
+    def test_terminate_connection(self):
+        """Test terminate_connection."""
+        result = self.driver.terminate_connection(self.vol, CONNECTOR)
+        self.assertIsNone(result)
+
+    def test_terminate_connection_without_connector(self):
+        """Test terminate_connection_without_connector."""
+        result = self.driver.terminate_connection(self.vol, None)
+        self.assertIsNone(result)
+
+
+@ddt.ddt
+class TestKaminarioFC(TestKaminarioCommon):
 
     def _setup_driver(self):
         self.driver = (kaminario_fc.
@@ -539,9 +618,25 @@ class TestKaminarioFC(TestKaminarioISCSI):
     def test_get_target_info(self):
         """Test get_target_info."""
         target_wwpn = self.driver.get_target_info(self.vol)
-        self.assertEqual(['50024f4053300300'], target_wwpn)
+        self.assertEqual(['50024f4053300301', '50024f4053300302'],
+                         target_wwpn)
 
     def test_terminate_connection(self):
         """Test terminate_connection."""
         result = self.driver.terminate_connection(self.vol, CONNECTOR)
         self.assertIn('data', result)
+
+    def test_terminate_connection_without_connector(self):
+        """Test terminate_connection_without_connector."""
+        result = self.driver.terminate_connection(self.vol, None)
+        self.assertIn('data', result)
+
+    @ddt.data(True, False)
+    def test_get_initiator_host_name_unique(self, in_shared):
+        cfg = self._set_unique_fqdn_override(False, in_shared)
+        self.mock_object(self.driver, 'configuration', cfg)
+        connector = CONNECTOR.copy()
+        del connector['initiator']
+        result = self.driver.get_initiator_host_name(connector)
+        expected = re.sub('[:.]', '_', connector['wwnns'][0][::-1][:32])
+        self.assertEqual(expected, result)
