@@ -10,11 +10,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import ddt
 import os
 import socket
+from unittest import mock
 
-import mock
+import ddt
 from oslo_concurrency import processutils
 from oslo_config import cfg
 
@@ -30,8 +30,8 @@ from cinder.tests.unit import utils as tests_utils
 from cinder.tests.unit.volume import test_driver
 from cinder.volume import configuration as conf
 from cinder.volume.drivers import lvm
-import cinder.volume.utils
-from cinder.volume import utils as volutils
+import cinder.volume.volume_utils
+from cinder.volume import volume_utils
 
 CONF = cfg.CONF
 
@@ -74,8 +74,8 @@ class LVMVolumeDriverTestCase(test_driver.BaseDriverTestCase):
         self.assertRaises(exception.VolumeBackendAPIException,
                           lvm_driver._delete_volume, volume)
 
-    @mock.patch.object(volutils, 'clear_volume')
-    @mock.patch.object(volutils, 'copy_volume')
+    @mock.patch.object(volume_utils, 'clear_volume')
+    @mock.patch.object(volume_utils, 'copy_volume')
     @mock.patch.object(fake_driver.FakeLoggingVolumeDriver, 'create_export')
     def test_delete_volume_thinlvm_snap(self, _mock_create_export,
                                         mock_copy, mock_clear):
@@ -97,7 +97,7 @@ class LVMVolumeDriverTestCase(test_driver.BaseDriverTestCase):
                          'size': 123}
         lvm_driver._delete_volume(fake_snapshot, is_snapshot=True)
 
-    @mock.patch.object(volutils, 'get_all_volume_groups',
+    @mock.patch.object(volume_utils, 'get_all_volume_groups',
                        return_value=[{'name': 'cinder-volumes'}])
     @mock.patch('cinder.brick.local_dev.lvm.LVM.get_lvm_version',
                 return_value=(2, 2, 100))
@@ -186,7 +186,7 @@ class LVMVolumeDriverTestCase(test_driver.BaseDriverTestCase):
 
         with mock.patch.object(self.volume.driver, 'vg'), \
                 mock.patch.object(self.volume.driver, '_create_volume'), \
-                mock.patch.object(volutils, 'copy_volume') as mock_copy:
+                mock.patch.object(volume_utils, 'copy_volume') as mock_copy:
 
             # Test case for thick LVM
             src_volume = tests_utils.create_volume(self.context)
@@ -244,7 +244,7 @@ class LVMVolumeDriverTestCase(test_driver.BaseDriverTestCase):
                                                    snapshot_ref)
             mock_extend.assert_called_with(dst_volume, dst_volume['size'])
 
-    @mock.patch.object(cinder.volume.utils, 'get_all_volume_groups',
+    @mock.patch.object(cinder.volume.volume_utils, 'get_all_volume_groups',
                        return_value=[{'name': 'cinder-volumes'}])
     @mock.patch('cinder.brick.local_dev.lvm.LVM.update_volume_group_info')
     @mock.patch('cinder.brick.local_dev.lvm.LVM.get_all_physical_volumes')
@@ -266,7 +266,7 @@ class LVMVolumeDriverTestCase(test_driver.BaseDriverTestCase):
 
         self.assertEqual('thin', lvm_driver.configuration.lvm_type)
 
-    @mock.patch.object(cinder.volume.utils, 'get_all_volume_groups',
+    @mock.patch.object(cinder.volume.volume_utils, 'get_all_volume_groups',
                        return_value=[{'name': 'cinder-volumes'}])
     @mock.patch.object(cinder.brick.local_dev.lvm.LVM, 'get_volumes',
                        return_value=[])
@@ -290,7 +290,7 @@ class LVMVolumeDriverTestCase(test_driver.BaseDriverTestCase):
 
         self.assertEqual('thin', lvm_driver.configuration.lvm_type)
 
-    @mock.patch.object(cinder.volume.utils, 'get_all_volume_groups',
+    @mock.patch.object(cinder.volume.volume_utils, 'get_all_volume_groups',
                        return_value=[{'name': 'cinder-volumes'}])
     @mock.patch('cinder.brick.local_dev.lvm.LVM.get_lv_info')
     @mock.patch('cinder.brick.local_dev.lvm.LVM.activate_lv')
@@ -310,7 +310,7 @@ class LVMVolumeDriverTestCase(test_driver.BaseDriverTestCase):
 
         self.assertEqual('default', lvm_driver.configuration.lvm_type)
 
-    @mock.patch.object(cinder.volume.utils, 'get_all_volume_groups',
+    @mock.patch.object(cinder.volume.volume_utils, 'get_all_volume_groups',
                        return_value=[{'name': 'cinder-volumes'}])
     @mock.patch('cinder.brick.local_dev.lvm.LVM.get_lv_info')
     @mock.patch('cinder.brick.local_dev.lvm.LVM.activate_lv')
@@ -394,7 +394,7 @@ class LVMVolumeDriverTestCase(test_driver.BaseDriverTestCase):
         self.assertFalse(moved)
         self.assertIsNone(model_update)
 
-    @mock.patch.object(volutils, 'get_all_volume_groups',
+    @mock.patch.object(volume_utils, 'get_all_volume_groups',
                        return_value=[{'name': 'cinder-volumes'}])
     def test_lvm_migrate_volume_same_volume_group(self, vgs):
         hostname = socket.gethostname()
@@ -414,9 +414,9 @@ class LVMVolumeDriverTestCase(test_driver.BaseDriverTestCase):
     @mock.patch.object(lvm.LVMVolumeDriver, '_create_volume')
     @mock.patch.object(brick_lvm.LVM, 'get_all_physical_volumes')
     @mock.patch.object(brick_lvm.LVM, 'delete')
-    @mock.patch.object(volutils, 'copy_volume',
+    @mock.patch.object(volume_utils, 'copy_volume',
                        side_effect=processutils.ProcessExecutionError)
-    @mock.patch.object(volutils, 'get_all_volume_groups',
+    @mock.patch.object(volume_utils, 'get_all_volume_groups',
                        return_value=[{'name': 'cinder-volumes'}])
     def test_lvm_migrate_volume_volume_copy_error(self, vgs, copy_volume,
                                                   mock_delete, mock_pvs,
@@ -434,7 +434,7 @@ class LVMVolumeDriverTestCase(test_driver.BaseDriverTestCase):
                           vol, host)
         mock_delete.assert_called_once_with(vol)
 
-    @mock.patch.object(volutils, 'get_all_volume_groups',
+    @mock.patch.object(volume_utils, 'get_all_volume_groups',
                        return_value=[{'name': 'cinder-volumes-2'}])
     def test_lvm_volume_group_missing(self, vgs):
         hostname = socket.gethostname()
@@ -475,8 +475,8 @@ class LVMVolumeDriverTestCase(test_driver.BaseDriverTestCase):
                                return_value = [{}]), \
                 mock.patch.object(self.volume.driver, '_execute') \
                 as mock_execute, \
-                mock.patch.object(volutils, 'copy_volume') as mock_copy, \
-                mock.patch.object(volutils, 'get_all_volume_groups',
+                mock.patch.object(volume_utils, 'copy_volume') as mock_copy, \
+                mock.patch.object(volume_utils, 'get_all_volume_groups',
                                   side_effect = get_all_volume_groups), \
                 mock.patch.object(self.volume.driver, '_delete_volume'):
 
@@ -523,8 +523,8 @@ class LVMVolumeDriverTestCase(test_driver.BaseDriverTestCase):
                                return_value = [{}]), \
                 mock.patch.object(lvm_driver, '_execute') \
                 as mock_execute, \
-                mock.patch.object(volutils, 'copy_volume') as mock_copy, \
-                mock.patch.object(volutils, 'get_all_volume_groups',
+                mock.patch.object(volume_utils, 'copy_volume') as mock_copy, \
+                mock.patch.object(volume_utils, 'get_all_volume_groups',
                                   side_effect = get_all_volume_groups), \
                 mock.patch.object(lvm_driver, '_delete_volume'):
 
@@ -610,11 +610,11 @@ class LVMVolumeDriverTestCase(test_driver.BaseDriverTestCase):
                                'rename_volume') as mock_rename_volume, \
                 mock.patch.object(self.volume.driver.vg, 'get_volume',
                                   self._get_manage_existing_lvs):
-                mock_rename_volume.return_value = _rename_volume
-                size = self.volume.driver.manage_existing_get_size(vol, ref)
-                self.assertEqual(2, size)
-                model_update = self.volume.driver.manage_existing(vol, ref)
-                self.assertIsNone(model_update)
+            mock_rename_volume.return_value = _rename_volume
+            size = self.volume.driver.manage_existing_get_size(vol, ref)
+            self.assertEqual(2, size)
+            model_update = self.volume.driver.manage_existing(vol, ref)
+            self.assertIsNone(model_update)
 
     def test_lvm_manage_existing_bad_size(self):
         """Make sure correct exception on bad size returned from LVM.
@@ -673,13 +673,13 @@ class LVMVolumeDriverTestCase(test_driver.BaseDriverTestCase):
                                'rename_volume') as mock_rename_volume, \
                 mock.patch.object(self.volume.driver.vg, 'get_volume',
                                   self._get_manage_existing_lvs):
-                mock_rename_volume.return_value = _rename_volume
-                size = self.volume.driver.manage_existing_snapshot_get_size(
-                    snp, ref)
-                self.assertEqual(2, size)
-                model_update = self.volume.driver.manage_existing_snapshot(
-                    snp, ref)
-                self.assertIsNone(model_update)
+            mock_rename_volume.return_value = _rename_volume
+            size = self.volume.driver.manage_existing_snapshot_get_size(
+                snp, ref)
+            self.assertEqual(2, size)
+            model_update = self.volume.driver.manage_existing_snapshot(
+                snp, ref)
+            self.assertIsNone(model_update)
 
     def test_lvm_manage_existing_snapshot_bad_ref(self):
         """Error case where specified LV snapshot doesn't exist.
@@ -1060,10 +1060,10 @@ class LVMISCSITestCase(test_driver.BaseDriverTestCase):
                                   'mountpoint': '/dev/vdc',
                                   'initiator': 'iqn.2012-07.org.fake:01'}
 
-        attachment1 = fake_volume.fake_volume_attachment_obj(self.context)
+        attachment1 = fake_volume.volume_attachment_ovo(self.context)
         attachment1.connector = connector_mountpoint_1
 
-        attachment2 = fake_volume.fake_volume_attachment_obj(self.context)
+        attachment2 = fake_volume.volume_attachment_ovo(self.context)
         attachment2.connector = connector_mountpoint_2
 
         # Create a multiattach volume object with two active attachments on
