@@ -74,7 +74,7 @@ class RemoteFsSnapDriverTestCase(test.TestCase):
         self._fake_snap_c = fake_snapshot.fake_snapshot_obj(self.context)
         self._fake_snap_c.volume = self.volume_c
         self.volume_c_path = os.path.join(self._FAKE_MNT_POINT,
-                                          self._fake_snap_c.name)
+                                          self.volume_c.name)
         self._fake_snap_c_path = (self.volume_c_path + '.' +
                                   self._fake_snap_c.id)
 
@@ -356,6 +356,18 @@ class RemoteFsSnapDriverTestCase(test.TestCase):
             snapshot.volume.status = 'backing-up'
             snapshot.volume.attach_status = 'attached'
             expected_method_called = '_create_snapshot_online'
+            conn_info = ('{"driver_volume_type": "nfs",'
+                         '"export": "localhost:/srv/nfs1",'
+                         '"name": "old_name"}')
+            attachment = fake_volume.volume_attachment_ovo(
+                self.context, connection_info=conn_info)
+            snapshot.volume.volume_attachment.objects.append(attachment)
+            mock_save = self.mock_object(attachment, 'save')
+
+            # After the snapshot the connection info should change the name of
+            # the file
+            expected = copy.deepcopy(attachment.connection_info)
+            expected['name'] = snapshot.volume.name + '.' + snapshot.id
         else:
             expected_method_called = '_do_create_snapshot'
 
@@ -372,6 +384,11 @@ class RemoteFsSnapDriverTestCase(test.TestCase):
             mock.sentinel.fake_info_path,
             expected_snapshot_info)
 
+        if volume_in_use:
+            mock_save.assert_called_once()
+            changed_fields = attachment.cinder_obj_get_changes()
+            self.assertEqual(expected, changed_fields['connection_info'])
+
     @ddt.data({'encryption': True}, {'encryption': False})
     def test_create_snapshot_volume_available(self, encryption):
         self._test_create_snapshot(encryption=encryption)
