@@ -130,9 +130,11 @@ class PowerMaxFCDriver(san.SanDriver, driver.FibreChannelDriver):
               - Use of snap id instead of generation (bp powermax-snapset-ids)
               - Support for Failover Abilities (bp/powermax-failover-abilities)
         4.3.1 - Fix non-temporary snapshot delete (#1887962)
+        4.3.2 - Fix for create snapshot (#1939139)
+        4.3.3 - Allow for case mismatch in SGs (bug #1929429)
     """
 
-    VERSION = "4.3.1"
+    VERSION = "4.3.3"
 
     # ThirdPartySystems wiki
     CI_WIKI_NAME = "EMC_VMAX_CI"
