@@ -127,10 +127,12 @@ class HPE3PARISCSIDriver(hpebasedriver.HPE3PARDriverBase):
         4.0.3 - Set proper backend on subsequent operation, after group
                 failover. bug #1773069
         4.0.4 - Added Peer Persistence feature
+        4.0.5 - Added Primera array check. bug #1849525
+        4.0.6 - Allow iSCSI support for Primera 4.2 onwards
 
     """
 
-    VERSION = "4.0.4"
+    VERSION = "4.0.6"
 
     # The name of the CI wiki page.
     CI_WIKI_NAME = "HPE_Storage_CI"
@@ -140,6 +142,21 @@ class HPE3PARISCSIDriver(hpebasedriver.HPE3PARDriverBase):
         self.protocol = 'iSCSI'
 
     def _do_setup(self, common):
+        client_obj = common.client
+        is_primera = client_obj.is_primera_array()
+        if is_primera:
+            api_version = client_obj.getWsApiVersion()
+            array_version = api_version['build']
+            LOG.debug("array version: %(version)s",
+                      {'version': array_version})
+            if array_version < 40200000:
+                err_msg = (_('The iSCSI driver is not supported for '
+                             'Primera %(version)s. It is supported '
+                             'for Primera 4.2 or higher versions.')
+                           % {'version': array_version})
+                LOG.error(err_msg)
+                raise NotImplementedError()
+
         self.iscsi_ips = {}
         common.client_login()
         try:
@@ -487,7 +504,7 @@ class HPE3PARISCSIDriver(hpebasedriver.HPE3PARDriverBase):
             if is_force_detach:
                 common.terminate_connection(volume, None, None)
             else:
-                hostname = common._safe_hostname(connector['host'])
+                hostname = common._safe_hostname(connector, self.configuration)
                 common.terminate_connection(
                     volume,
                     hostname,
@@ -502,7 +519,7 @@ class HPE3PARISCSIDriver(hpebasedriver.HPE3PARDriverBase):
 
         Ignore exceptions caused by the keys not being present on a volume.
         """
-        vol_name = common._get_3par_vol_name(volume['id'])
+        vol_name = common._get_3par_vol_name(volume)
 
         try:
             common.client.removeVolumeMetaData(vol_name, CHAP_USER_KEY)
@@ -593,7 +610,7 @@ class HPE3PARISCSIDriver(hpebasedriver.HPE3PARDriverBase):
         domain = None
         username = None
         password = None
-        hostname = common._safe_hostname(connector['host'])
+        hostname = common._safe_hostname(connector, self.configuration)
 
         if remote_target:
             cpg = common._get_cpg_from_cpg_map(
@@ -608,7 +625,7 @@ class HPE3PARISCSIDriver(hpebasedriver.HPE3PARDriverBase):
         if not remote_target:
             # Get the CHAP secret if CHAP is enabled
             if common._client_conf['hpe3par_iscsi_chap_enabled']:
-                vol_name = common._get_3par_vol_name(volume['id'])
+                vol_name = common._get_3par_vol_name(volume)
                 username = common.client.getVolumeMetaData(
                     vol_name, CHAP_USER_KEY)['value']
                 password = common.client.getVolumeMetaData(
@@ -715,7 +732,7 @@ class HPE3PARISCSIDriver(hpebasedriver.HPE3PARDriverBase):
                             "Generating new CHAP key.")
 
         # Add CHAP credentials to the volume metadata
-        vol_name = common._get_3par_vol_name(volume['id'])
+        vol_name = common._get_3par_vol_name(volume)
         common.client.setVolumeMetaData(
             vol_name, CHAP_USER_KEY, chap_username)
         common.client.setVolumeMetaData(
@@ -742,7 +759,7 @@ class HPE3PARISCSIDriver(hpebasedriver.HPE3PARDriverBase):
         """
         common = self._login()
         try:
-            vol_name = common._get_3par_vol_name(volume['id'])
+            vol_name = common._get_3par_vol_name(volume)
             common.client.getVolume(vol_name)
         except hpeexceptions.HTTPNotFound:
             LOG.error("Volume %s doesn't exist on array.", vol_name)
