@@ -37,13 +37,13 @@ from oslo_utils import units
 import six
 
 from cinder import exception
-from cinder.i18n import _, _LE, _LI, _LW
+from cinder.i18n import _
 from cinder import utils
 from cinder.volume.drivers.netapp.dataontap.client import api as netapp_api
 from cinder.volume.drivers.netapp.dataontap.utils import loopingcalls
 from cinder.volume.drivers.netapp import options as na_opts
 from cinder.volume.drivers.netapp import utils as na_utils
-from cinder.volume import utils as volume_utils
+from cinder.volume import volume_utils
 from cinder.zonemanager import utils as fczm_utils
 
 LOG = logging.getLogger(__name__)
@@ -115,7 +115,9 @@ class NetAppBlockStorageLibrary(object):
             na_opts.netapp_provisioning_opts)
         self.configuration.append_config_values(na_opts.netapp_san_opts)
         self.max_over_subscription_ratio = (
-            self.configuration.max_over_subscription_ratio)
+            volume_utils.get_max_over_subscription_ratio(
+                self.configuration.max_over_subscription_ratio,
+                supports_auto=True))
         self.reserved_percentage = self._get_reserved_percentage()
         self.loopingcalls = loopingcalls.LoopingCalls()
 
@@ -134,10 +136,10 @@ class NetAppBlockStorageLibrary(object):
         divisor = self.configuration.netapp_size_multiplier
         reserved_ratio = round(1 - (1 / divisor), 2)
         reserved_percentage = 100 * int(reserved_ratio)
-        msg = _LW('The "netapp_size_multiplier" configuration option is '
-                  'deprecated and will be removed in the Mitaka release. '
-                  'Please set "reserved_percentage = %d" instead.') % (
-                      reserved_percentage)
+        msg = ('The "netapp_size_multiplier" configuration option is '
+               'deprecated and will be removed in the Mitaka release. '
+               'Please set "reserved_percentage = %d" instead.') % (
+                   reserved_percentage)
         versionutils.report_deprecated_feature(LOG, msg)
         return reserved_percentage
 
@@ -161,12 +163,12 @@ class NetAppBlockStorageLibrary(object):
             msg = _("Invalid value for NetApp configuration"
                     " option netapp_lun_ostype.")
             LOG.error(msg)
-            raise exception.NetAppDriverException(msg)
+            raise na_utils.NetAppDriverException(msg)
         if self.host_type not in self.ALLOWED_IGROUP_HOST_TYPES:
             msg = _("Invalid value for NetApp configuration"
                     " option netapp_host_type.")
             LOG.error(msg)
-            raise exception.NetAppDriverException(msg)
+            raise na_utils.NetAppDriverException(msg)
         lun_list = self.zapi_client.get_lun_list()
         self._extract_and_populate_luns(lun_list)
         LOG.debug("Success getting list of LUNs from server.")
@@ -177,20 +179,28 @@ class NetAppBlockStorageLibrary(object):
 
         Inheriting class overrides and then explicitly calls this method.
         """
+
         # Add the task that deletes snapshots marked for deletion.
         self.loopingcalls.add_task(
             self._delete_snapshots_marked_for_deletion,
             loopingcalls.ONE_MINUTE,
             loopingcalls.ONE_MINUTE)
 
+        # Add the task that logs EMS messages
+        self.loopingcalls.add_task(
+            self._handle_ems_logging,
+            loopingcalls.ONE_HOUR)
+
     def _delete_snapshots_marked_for_deletion(self):
-        volume_list = self._get_backing_flexvol_names()
-        snapshots = self.zapi_client.get_snapshots_marked_for_deletion(
-            volume_list)
+        snapshots = self.zapi_client.get_snapshots_marked_for_deletion()
         for snapshot in snapshots:
             self.zapi_client.delete_snapshot(
                 snapshot['volume_name'], snapshot['name'])
 
+    def _handle_ems_logging(self):
+        """Log autosupport messages."""
+        raise NotImplementedError()
+
     def get_pool(self, volume):
         """Return pool name where volume resides.
 
@@ -227,13 +237,15 @@ class NetAppBlockStorageLibrary(object):
         qos_policy_group_name = (
             na_utils.get_qos_policy_group_name_from_info(
                 qos_policy_group_info))
+        qos_policy_group_is_adaptive = volume_utils.is_boolean_str(
+            extra_specs.get('netapp:qos_policy_group_is_adaptive'))
 
         try:
             self._create_lun(pool_name, lun_name, size, metadata,
-                             qos_policy_group_name)
+                             qos_policy_group_name,
+                             qos_policy_group_is_adaptive)
         except Exception:
-            LOG.exception(_LE("Exception creating LUN %(name)s in pool "
-                              "%(pool)s."),
+            LOG.exception("Exception creating LUN %(name)s in pool %(pool)s.",
                           {'name': lun_name, 'pool': pool_name})
             self._mark_qos_policy_group_for_deletion(qos_policy_group_info)
             msg = _("Volume %s could not be created.")
@@ -249,9 +261,17 @@ class NetAppBlockStorageLibrary(object):
         handle = self._create_lun_handle(metadata)
         self._add_lun_to_table(NetAppLun(handle, lun_name, size, metadata))
 
+        model_update = self._get_volume_model_update(volume)
+
+        return model_update
+
     def _setup_qos_for_volume(self, volume, extra_specs):
         return None
 
+    def _get_volume_model_update(self, volume):
+        """Provide any updates necessary for a volume being created/managed."""
+        raise NotImplementedError
+
     def _mark_qos_policy_group_for_deletion(self, qos_policy_group_info):
         return
 
@@ -264,11 +284,19 @@ class NetAppBlockStorageLibrary(object):
 
         metadata = self._get_lun_attr(lun_name, 'metadata')
         if metadata:
-            self.zapi_client.destroy_lun(metadata['Path'])
+            try:
+                self.zapi_client.destroy_lun(metadata['Path'])
+            except netapp_api.NaApiError as e:
+                if e.code == netapp_api.EOBJECTNOTFOUND:
+                    LOG.warning("Failure deleting LUN %(name)s. %(message)s",
+                                {'name': lun_name, 'message': e})
+                else:
+                    error_message = (_('A NetApp Api Error occurred: %s') % e)
+                    raise na_utils.NetAppDriverException(error_message)
             self.lun_table.pop(lun_name)
         else:
-            LOG.warning(_LW("No entry in LUN table for volume/snapshot"
-                            " %(name)s."), {'name': lun_name})
+            LOG.warning("No entry in LUN table for volume/snapshot"
+                        " %(name)s.", {'name': lun_name})
 
     def ensure_export(self, context, volume):
         """Driver entry point to get the export info for an existing volume."""
@@ -295,7 +323,9 @@ class NetAppBlockStorageLibrary(object):
         This driver implements snapshots by using efficient single-file
         (LUN) cloning.
         """
+        self._create_snapshot(snapshot)
 
+    def _create_snapshot(self, snapshot):
         vol_name = snapshot['volume_name']
         snapshot_name = snapshot['name']
         lun = self._get_lun_from_table(vol_name)
@@ -330,11 +360,15 @@ class NetAppBlockStorageLibrary(object):
         qos_policy_group_name = (
             na_utils.get_qos_policy_group_name_from_info(
                 qos_policy_group_info))
+        qos_policy_group_is_adaptive = volume_utils.is_boolean_str(
+            extra_specs.get('netapp:qos_policy_group_is_adaptive'))
 
         try:
-            self._clone_lun(source_name, destination_name,
-                            space_reserved=self.lun_space_reservation,
-                            qos_policy_group_name=qos_policy_group_name)
+            self._clone_lun(
+                source_name, destination_name,
+                space_reserved=self.lun_space_reservation,
+                qos_policy_group_name=qos_policy_group_name,
+                qos_policy_group_is_adaptive=qos_policy_group_is_adaptive)
 
             if destination_size != source_size:
 
@@ -344,14 +378,15 @@ class NetAppBlockStorageLibrary(object):
                                         qos_policy_group_name)
                 except Exception:
                     with excutils.save_and_reraise_exception():
-                        LOG.error(
-                            _LE("Resizing %s failed. Cleaning volume."),
-                            destination_volume['id'])
+                        LOG.error("Resizing %s failed. Cleaning volume.",
+                                  destination_volume['id'])
                         self.delete_volume(destination_volume)
 
+            return self._get_volume_model_update(destination_volume)
+
         except Exception:
-            LOG.exception(_LE("Exception cloning volume %(name)s from source "
-                          "volume %(source)s."),
+            LOG.exception("Exception cloning volume %(name)s from source "
+                          "volume %(source)s.",
                           {'name': destination_name, 'source': source_name})
 
             self._mark_qos_policy_group_for_deletion(qos_policy_group_info)
@@ -393,11 +428,11 @@ class NetAppBlockStorageLibrary(object):
         igroup_name, ig_host_os, ig_type = self._get_or_create_igroup(
             initiator_list, initiator_type, self.host_type)
         if ig_host_os != self.host_type:
-            LOG.warning(_LW("LUN misalignment may occur for current"
-                            " initiator group %(ig_nm)s) with host OS type"
-                            " %(ig_os)s. Please configure initiator group"
-                            " manually according to the type of the"
-                            " host OS."),
+            LOG.warning("LUN misalignment may occur for current"
+                        " initiator group %(ig_nm)s) with host OS type"
+                        " %(ig_os)s. Please configure initiator group"
+                        " manually according to the type of the"
+                        " host OS.",
                         {'ig_nm': igroup_name, 'ig_os': ig_host_os})
         try:
             return self.zapi_client.map_lun(path, igroup_name, lun_id=lun_id)
@@ -412,9 +447,19 @@ class NetAppBlockStorageLibrary(object):
 
     def _unmap_lun(self, path, initiator_list):
         """Unmaps a LUN from given initiator."""
-        (igroup_name, _lun_id) = self._find_mapped_lun_igroup(path,
-                                                              initiator_list)
-        self.zapi_client.unmap_lun(path, igroup_name)
+
+        if len(initiator_list) != 0:
+            lun_unmap_list = []
+            (igroup_name, _) = self._find_mapped_lun_igroup(
+                path, initiator_list)
+            lun_unmap_list.append((path, igroup_name))
+        else:
+            lun_maps = self.zapi_client.get_lun_map(path)
+            lun_unmap_list = [(path, lun_m['initiator-group'])
+                              for lun_m in lun_maps]
+
+        for _path, _igroup_name in lun_unmap_list:
+            self.zapi_client.unmap_lun(_path, _igroup_name)
 
     def _find_mapped_lun_igroup(self, path, initiator_list):
         """Find an igroup for a LUN mapped to the given initiator(s)."""
@@ -488,9 +533,9 @@ class NetAppBlockStorageLibrary(object):
             attr = getattr(self._get_lun_from_table(name), attr)
             return attr
         except exception.VolumeNotFound as e:
-            LOG.error(_LE("Message: %s"), e.msg)
+            LOG.error("Message: %s", e.msg)
         except Exception as e:
-            LOG.error(_LE("Error getting LUN attribute. Exception: %s"), e)
+            LOG.error("Error getting LUN attribute. Exception: %s", e)
         return None
 
     def _create_lun_meta(self, lun):
@@ -554,20 +599,35 @@ class NetAppBlockStorageLibrary(object):
         new_size_bytes = six.text_type(int(new_size) * units.Gi)
         # Reused by clone scenarios.
         # Hence comparing the stored size.
-        if curr_size_bytes != new_size_bytes:
+        if curr_size_bytes == new_size_bytes:
+            LOG.info("No need to extend volume %s"
+                     " as it is already the requested new size.", name)
+            return
+
+        ontap_version = self.zapi_client.get_ontap_version(cached=True)
+
+        if ontap_version >= '9.5':
+            self.zapi_client.do_direct_resize(path, new_size_bytes)
+        else:
             lun_geometry = self.zapi_client.get_lun_geometry(path)
-            if (lun_geometry and lun_geometry.get("max_resize")
-                    and int(lun_geometry.get("max_resize")) >=
+            if (lun_geometry and int(lun_geometry.get("max_resize", 0)) >=
                     int(new_size_bytes)):
                 self.zapi_client.do_direct_resize(path, new_size_bytes)
             else:
+                if volume['attach_status'] != 'detached':
+                    msg = _('Volume %(vol_id)s cannot be resized from '
+                            '%(old_size)s to %(new_size)s, because would '
+                            'exceed its max geometry %(max_geo)s while '
+                            'not being detached.')
+                    raise exception.VolumeBackendAPIException(data=msg % {
+                        'vol_id': name,
+                        'old_size': curr_size_bytes,
+                        'new_size': new_size_bytes,
+                        'max_geo': lun_geometry.get("max_resize")})
                 self._do_sub_clone_resize(
                     path, new_size_bytes,
                     qos_policy_group_name=qos_policy_group_name)
-            self.lun_table[name].size = new_size_bytes
-        else:
-            LOG.info(_LI("No need to extend volume %s"
-                         " as it is already the requested new size."), name)
+        self.lun_table[name].size = new_size_bytes
 
     def _get_vol_option(self, volume_name, option_name):
         """Get the value for the volume option."""
@@ -586,7 +646,7 @@ class NetAppBlockStorageLibrary(object):
         Clones the block ranges, swaps the LUNs, and deletes the source LUN.
         """
         seg = lun_path.split("/")
-        LOG.info(_LI("Resizing LUN %s using clone operation."), seg[-1])
+        LOG.info("Resizing LUN %s using clone operation.", seg[-1])
         lun_name = seg[-1]
         vol_name = seg[2]
         lun = self._get_lun_from_table(lun_name)
@@ -620,7 +680,7 @@ class NetAppBlockStorageLibrary(object):
         """Try post sub clone resize in a transactional manner."""
         st_tm_mv, st_nw_mv, st_del_old = None, None, None
         seg = path.split("/")
-        LOG.info(_LI("Post clone resize LUN %s"), seg[-1])
+        LOG.info("Post clone resize LUN %s", seg[-1])
         new_lun = 'new-%s' % (seg[-1])
         tmp_lun = 'tmp-%s' % (seg[-1])
         tmp_path = "/vol/%s/%s" % (seg[2], tmp_lun)
@@ -640,12 +700,12 @@ class NetAppBlockStorageLibrary(object):
                     raise exception.VolumeBackendAPIException(
                         data=msg % (seg[-1]))
                 elif st_del_old is None:
-                    LOG.error(_LE("Failure deleting staged tmp LUN %s."),
+                    LOG.error("Failure deleting staged tmp LUN %s.",
                               tmp_lun)
                 else:
-                    LOG.error(_LE("Unknown exception in"
-                                  " post clone resize LUN %s."), seg[-1])
-                    LOG.error(_LE("Exception details: %s"), e)
+                    LOG.error("Unknown exception in"
+                              " post clone resize LUN %s.", seg[-1])
+                    LOG.error("Exception details: %s", e)
 
     def _get_lun_block_count(self, path):
         """Gets block counts for the LUN."""
@@ -686,23 +746,26 @@ class NetAppBlockStorageLibrary(object):
         path = lun.get_metadata_property('Path')
         if lun.name == volume['name']:
             new_path = path
-            LOG.info(_LI("LUN with given ref %s need not be renamed "
-                         "during manage operation."), existing_ref)
+            LOG.info("LUN with given ref %s need not be renamed "
+                     "during manage operation.", existing_ref)
         else:
             (rest, splitter, name) = path.rpartition('/')
             new_path = '%s/%s' % (rest, volume['name'])
             self.zapi_client.move_lun(path, new_path)
             lun = self._get_existing_vol_with_manage_ref(
                 {'source-name': new_path})
+
         if qos_policy_group_name is not None:
             self.zapi_client.set_lun_qos_policy_group(new_path,
                                                       qos_policy_group_name)
         self._add_lun_to_table(lun)
-        LOG.info(_LI("Manage operation completed for LUN with new path"
-                     " %(path)s and uuid %(uuid)s."),
+        LOG.info("Manage operation completed for LUN with new path"
+                 " %(path)s and uuid %(uuid)s.",
                  {'path': lun.get_metadata_property('Path'),
                   'uuid': lun.get_metadata_property('UUID')})
 
+        return self._get_volume_model_update(volume)
+
     def manage_existing_get_size(self, volume, existing_ref):
         """Return size of volume to be managed by manage_existing.
 
@@ -754,8 +817,8 @@ class NetAppBlockStorageLibrary(object):
            Does not delete the underlying backend storage object.
         """
         managed_lun = self._get_lun_from_table(volume['name'])
-        LOG.info(_LI("Unmanaged LUN with current path %(path)s and uuid "
-                     "%(uuid)s."),
+        LOG.info("Unmanaged LUN with current path %(path)s and uuid "
+                 "%(uuid)s.",
                  {'path': managed_lun.get_metadata_property('Path'),
                   'uuid': managed_lun.get_metadata_property('UUID')
                   or 'unknown'})
@@ -788,13 +851,12 @@ class NetAppBlockStorageLibrary(object):
                   "initiator %(initiator_name)s",
                   {'name': name, 'initiator_name': initiator_name})
 
-        preferred_target = self._get_preferred_target_from_list(
-            target_list)
-        if preferred_target is None:
+        targets = self._get_targets_from_list(target_list)
+        if not targets:
             msg = _('Failed to get target portal for the LUN %s')
             raise exception.VolumeBackendAPIException(data=msg % name)
-        (address, port) = (preferred_target['address'],
-                           preferred_target['port'])
+        addresses = [target['address'] for target in targets]
+        ports = [target['port'] for target in targets]
 
         iqn = self.zapi_client.get_iscsi_service_details()
         if not iqn:
@@ -802,8 +864,8 @@ class NetAppBlockStorageLibrary(object):
             raise exception.VolumeBackendAPIException(data=msg % name)
 
         properties = na_utils.get_iscsi_connection_properties(lun_id, volume,
-                                                              iqn, address,
-                                                              port)
+                                                              iqn, addresses,
+                                                              ports)
 
         if self.configuration.use_chap_auth:
             chap_username, chap_password = self._configure_chap(initiator_name)
@@ -830,18 +892,29 @@ class NetAppBlockStorageLibrary(object):
         properties['data']['discovery_auth_username'] = username
         properties['data']['discovery_auth_password'] = password
 
-    def _get_preferred_target_from_list(self, target_details_list,
-                                        filter=None):
-        preferred_target = None
-        for target in target_details_list:
-            if filter and target['address'] not in filter:
-                continue
-            if target.get('interface-enabled', 'true') == 'true':
-                preferred_target = target
-                break
-        if preferred_target is None and len(target_details_list) > 0:
-            preferred_target = target_details_list[0]
-        return preferred_target
+    def _get_targets_from_list(self, target_details_list, filter=None):
+        targets = [target for target in target_details_list
+                   if (not filter or target['address'] in filter) and
+                   target.get('interface-enabled', 'true') == 'true']
+        if not targets and len(target_details_list) > 0:
+            targets = target_details_list
+        return targets
+
+    def _is_multiattached(self, volume, connector):
+        """Returns whether the volume is multiattached.
+
+        Returns True if the volume is attached to multiple instances using the
+        same initiator as the given one.
+        Returns False otherwise.
+        """
+        if not volume.multiattach or not volume.volume_attachment:
+            return False
+
+        same_connector = (True for at in volume.volume_attachment
+                          if at.connector and
+                          at.connector['initiator'] == connector['initiator'])
+        next(same_connector, False)
+        return next(same_connector, False)
 
     def terminate_connection_iscsi(self, volume, connector, **kwargs):
         """Driver entry point to unattach a volume from an instance.
@@ -849,15 +922,24 @@ class NetAppBlockStorageLibrary(object):
         Unmask the LUN on the storage system so the given initiator can no
         longer access it.
         """
+        if connector and self._is_multiattached(volume, connector):
+            return
 
-        initiator_name = connector['initiator']
         name = volume['name']
+        if connector is None:
+            initiators = []
+            LOG.debug('Unmapping LUN %(name)s from all initiators',
+                      {'name': name})
+        else:
+            initiators = [connector['initiator']]
+            LOG.debug("Unmapping LUN %(name)s from the initiator "
+                      "%(initiator_name)s", {'name': name,
+                                             'initiator_name': initiators})
+
         metadata = self._get_lun_attr(name, 'metadata')
         path = metadata['Path']
-        self._unmap_lun(path, [initiator_name])
-        LOG.debug("Unmapped LUN %(name)s from the initiator "
-                  "%(initiator_name)s",
-                  {'name': name, 'initiator_name': initiator_name})
+
+        self._unmap_lun(path, initiators)
 
     def initialize_connection_fc(self, volume, connector):
         """Initializes the connection and returns connection info.
@@ -870,10 +952,10 @@ class NetAppBlockStorageLibrary(object):
         correspond to the list of remote wwn(s) that will export the volume.
         Example return values:
 
-        .. code-block:: json
+        .. code-block:: default
 
             {
-                'driver_volume_type': 'fibre_channel'
+                'driver_volume_type': 'fibre_channel',
                 'data': {
                     'target_discovered': True,
                     'target_lun': 1,
@@ -885,10 +967,12 @@ class NetAppBlockStorageLibrary(object):
                 }
             }
 
-            or
+        Or
+
+        .. code-block:: default
 
              {
-                'driver_volume_type': 'fibre_channel'
+                'driver_volume_type': 'fibre_channel',
                 'data': {
                     'target_discovered': True,
                     'target_lun': 1,
@@ -947,24 +1031,29 @@ class NetAppBlockStorageLibrary(object):
                          an empty dict for the 'data' key
         """
 
-        initiators = [fczm_utils.get_formatted_wwn(wwpn)
-                      for wwpn in connector['wwpns']]
         name = volume['name']
+        if connector is None:
+            initiators = []
+            LOG.debug('Unmapping LUN %(name)s from all initiators',
+                      {'name': name})
+        else:
+            initiators = [fczm_utils.get_formatted_wwn(wwpn)
+                          for wwpn in connector['wwpns']]
+            LOG.debug("Unmapping LUN %(name)s from the initiators "
+                      "%(initiator_name)s", {'name': name,
+                                             'initiator_name': initiators})
+
         metadata = self._get_lun_attr(name, 'metadata')
         path = metadata['Path']
 
         self._unmap_lun(path, initiators)
 
-        LOG.debug("Unmapped LUN %(name)s from the initiator %(initiators)s",
-                  {'name': name, 'initiators': initiators})
-
         info = {'driver_volume_type': 'fibre_channel',
                 'data': {}}
 
-        if not self._has_luns_mapped_to_initiators(initiators):
+        if connector and not self._has_luns_mapped_to_initiators(initiators):
             # No more exports for this host, so tear down zone.
-            LOG.info(_LI("Need to remove FC Zone, building initiator "
-                         "target map"))
+            LOG.info("Need to remove FC Zone, building initiator target map")
 
             target_wwpns, initiator_target_map, num_paths = (
                 self._build_initiator_target_map(connector))
@@ -1013,132 +1102,6 @@ class NetAppBlockStorageLibrary(object):
 
         return target_wwpns, init_targ_map, num_paths
 
-    def create_consistencygroup(self, group):
-        """Driver entry point for creating a consistency group.
-
-        ONTAP does not maintain an actual CG construct. As a result, no
-        communication to the backend is necessary for consistency group
-        creation.
-
-        :return: Hard-coded model update for consistency group model.
-        """
-        model_update = {'status': 'available'}
-        return model_update
-
-    def delete_consistencygroup(self, group, volumes):
-        """Driver entry point for deleting a consistency group.
-
-        :return: Updated consistency group model and list of volume models
-                 for the volumes that were deleted.
-        """
-        model_update = {'status': 'deleted'}
-        volumes_model_update = []
-        for volume in volumes:
-            try:
-                self._delete_lun(volume['name'])
-                volumes_model_update.append(
-                    {'id': volume['id'], 'status': 'deleted'})
-            except Exception:
-                volumes_model_update.append(
-                    {'id': volume['id'], 'status': 'error_deleting'})
-                LOG.exception(_LE("Volume %(vol)s in the consistency group "
-                                  "could not be deleted."), {'vol': volume})
-        return model_update, volumes_model_update
-
-    def update_consistencygroup(self, group, add_volumes=None,
-                                remove_volumes=None):
-        """Driver entry point for updating a consistency group.
-
-        Since no actual CG construct is ever created in ONTAP, it is not
-        necessary to update any metadata on the backend. Since this is a NO-OP,
-        there is guaranteed to be no change in any of the volumes' statuses.
-        """
-        return None, None, None
-
-    def create_cgsnapshot(self, cgsnapshot, snapshots):
-        """Creates a Cinder cgsnapshot object.
-
-        The Cinder cgsnapshot object is created by making use of an
-        ephemeral ONTAP CG in order to provide write-order consistency for a
-        set of flexvol snapshots. First, a list of the flexvols backing the
-        given Cinder CG must be gathered. An ONTAP cg-snapshot of these
-        flexvols will create a snapshot copy of all the Cinder volumes in the
-        CG group. For each Cinder volume in the CG, it is then necessary to
-        clone its backing LUN from the ONTAP cg-snapshot. The naming convention
-        used for the clones is what indicates the clone's role as a Cinder
-        snapshot and its inclusion in a Cinder CG. The ONTAP CG-snapshot of
-        the flexvols is no longer required after having cloned the LUNs
-        backing the Cinder volumes in the Cinder CG.
-
-        :return: An implicit update for cgsnapshot and snapshots models that
-                 is interpreted by the manager to set their models to
-                 available.
-        """
-        flexvols = set()
-        for snapshot in snapshots:
-            flexvols.add(volume_utils.extract_host(snapshot['volume']['host'],
-                                                   level='pool'))
-
-        self.zapi_client.create_cg_snapshot(flexvols, cgsnapshot['id'])
-
-        for snapshot in snapshots:
-            self._clone_lun(snapshot['volume']['name'], snapshot['name'],
-                            source_snapshot=cgsnapshot['id'])
-
-        for flexvol in flexvols:
-            try:
-                self.zapi_client.wait_for_busy_snapshot(
-                    flexvol, cgsnapshot['id'])
-                self.zapi_client.delete_snapshot(
-                    flexvol, cgsnapshot['id'])
-            except exception.SnapshotIsBusy:
-                self.zapi_client.mark_snapshot_for_deletion(
-                    flexvol, cgsnapshot['id'])
-
-        return None, None
-
-    def delete_cgsnapshot(self, cgsnapshot, snapshots):
-        """Delete LUNs backing each snapshot in the cgsnapshot.
-
-        :return: An implicit update for snapshots models that is interpreted
-                 by the manager to set their models to deleted.
-        """
-        for snapshot in snapshots:
-            self._delete_lun(snapshot['name'])
-            LOG.debug("Snapshot %s deletion successful", snapshot['name'])
-
-        return None, None
-
-    def create_consistencygroup_from_src(self, group, volumes,
-                                         cgsnapshot=None, snapshots=None,
-                                         source_cg=None, source_vols=None):
-        """Creates a CG from a either a cgsnapshot or group of cinder vols.
-
-        :return: An implicit update for the volumes model that is
-                 interpreted by the manager as a successful operation.
-        """
-        LOG.debug("VOLUMES %s ", [dict(vol) for vol in volumes])
-
-        if cgsnapshot:
-            vols = zip(volumes, snapshots)
-
-            for volume, snapshot in vols:
-                source = {
-                    'name': snapshot['name'],
-                    'size': snapshot['volume_size'],
-                }
-                self._clone_source_to_destination(source, volume)
-
-        else:
-            vols = zip(volumes, source_vols)
-
-            for volume, old_src_vref in vols:
-                src_lun = self._get_lun_from_table(old_src_vref['name'])
-                source = {'name': src_lun.name, 'size': old_src_vref['size']}
-                self._clone_source_to_destination(source, volume)
-
-        return None, None
-
     def _get_backing_flexvol_names(self):
         """Returns a list of backing flexvol names."""
         raise NotImplementedError()
