@@ -468,12 +468,14 @@ class Client(client_base.Client):
             else:
                 block_count = zbc
 
+            is_sub_clone = block_count > 0
             zapi_args = {
                 'volume': volume,
                 'source-path': name,
                 'destination-path': new_name,
-                'space-reserve': space_reserved,
             }
+            if not is_sub_clone:
+                zapi_args['space-reserve'] = space_reserved
             if source_snapshot:
                 zapi_args['snapshot-name'] = source_snapshot
             if is_snapshot and self.features.BACKUP_CLONE_PARAM:
@@ -488,7 +490,7 @@ class Client(client_base.Client):
                 else:
                     clone_create.add_new_child('qos-policy-group-name',
                                                qos_policy_group_name)
-            if block_count > 0:
+            if is_sub_clone:
                 block_ranges = netapp_api.NaElement("block-ranges")
                 segments = int(math.ceil(block_count / float(bc_limit)))
                 bc = block_count
