@@ -20,7 +20,9 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import fileutils
+from oslo_utils import netutils
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
 import six
 import taskflow.engines
 from taskflow.patterns import linear_flow
@@ -656,6 +658,34 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
         self.db.volume_glance_metadata_bulk_create(context, volume_id,
                                                    volume_metadata)
 
+    @staticmethod
+    def _extract_cinder_ids(urls):
+        """Process a list of location URIs from glance
+
+        :param urls: list of glance location URIs
+        :return: list of IDs extracted from the 'cinder://' URIs
+
+        """
+        ids = []
+        for url in urls:
+            # The url can also be None and a TypeError is raised
+            # TypeError: a bytes-like object is required, not 'str'
+            if not url:
+                continue
+            parts = netutils.urlsplit(url)
+            if parts.scheme == 'cinder':
+                if parts.path:
+                    vol_id = parts.path.split('/')[-1]
+                else:
+                    vol_id = parts.netloc
+                if uuidutils.is_uuid_like(vol_id):
+                    ids.append(vol_id)
+                else:
+                    LOG.debug("Ignoring malformed image location uri "
+                              "'%(url)s'", {'url': url})
+
+        return ids
+
     def _clone_image_volume(self, context, volume, image_location, image_meta):
         """Create a volume efficiently from an existing image.
 
@@ -675,9 +705,9 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
 
         image_volume = None
         direct_url, locations = image_location
-        urls = set([direct_url] + [loc.get('url') for loc in locations or []])
-        image_volume_ids = [url[9:] for url in urls
-                            if url and url.startswith('cinder://')]
+        urls = list(set([direct_url]
+                        + [loc.get('url') for loc in locations or []]))
+        image_volume_ids = self._extract_cinder_ids(urls)
         image_volumes = self.db.volume_get_all_by_host(
             context, volume['host'], filters={'id': image_volume_ids})
 
